#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/base/colours_for_base/colours_for_base.rb'
# include ::Bioroebe::ColoursForBase
# =========================================================================== #
module Bioroebe

module ColoursForBase # === Bioroebe::ColoursForBase

  require 'bioroebe/colours/colours.rb'
  require 'bioroebe/colours/rev.rb'

  # ========================================================================= #
  # === ARRAY_HTML_COLOURS_IN_USE
  #
  # Next, we will define the KONSOLE Colours:
  # ========================================================================= #
  ARRAY_HTML_COLOURS_IN_USE = %w(
    burlywood
    cadetblue
    crimson
    cyan
    darkolivegreen
    deepskyblue
    forestgreen
    gold
    green
    lightblue
    lightgreen
    lightslategray
    lightsteelblue
    mediumslateblue
    mediumvioletred
    mediumturquoise
    mediumspringgreen
    olivedrab
    orangered
    olive
    orange
    orchid
    palevioletred
    palegoldenrod
    powderblue
    plum
    rosybrown
    royalblue
    salmon
    seagreen
    skyblue
    slateblue
    slategrey
    springgreen
    steelblue
    tomato
    mediumorchid
    mediumpurple
    mediumseagreen
    paleturquoise
    palegreen
    peru
    sienna
    teal
    turquoise
    violet
    yellow
    yellowgreen
  )

  # ========================================================================= #
  # === use_colours_within_the_bioroebe_namespace?
  #
  # Query whether we use colours in the whole Bioroebe namespace.
  # ========================================================================= #
  def use_colours_within_the_bioroebe_namespace?
    ::Bioroebe.use_colours?
  end

  # ========================================================================= #
  # === set_use_colours
  # ========================================================================= #
  def set_use_colours(i)
    @use_colours = i
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === egold
  # ========================================================================= #
  def egold(i = '')
    i = ::Bioroebe.gold(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    e "#{i}#{rev}"
  end

  # ========================================================================= #
  # === eorange
  # ========================================================================= #
  def eorange(i = '')
    i = ::Bioroebe.orange(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    e "#{i}#{rev}"
  end

  # ========================================================================= #
  # === colourize_this_aminoacid_sequence_for_the_commandline
  # ========================================================================= #
  def colourize_this_aminoacid_sequence_for_the_commandline(i)
    ::Bioroebe.colourize_this_aminoacid_sequence_for_the_commandline(i)
  end; alias colourize_this_aminoacid_sequence colourize_this_aminoacid_sequence_for_the_commandline # === colourize_this_aminoacid_sequence

  # ========================================================================= #
  # === KDE colours
  #
  # Define the KDE colours in use next.
  # ========================================================================= #
  ARRAY_HTML_COLOURS_IN_USE.each {|this_colour|
    define_method(this_colour.to_sym) {|i = ''|
      if use_colours? and use_colours_within_the_bioroebe_namespace?
        return Colours.send(this_colour.to_sym, i)
      end
      i
    }
  }

  # ========================================================================= #
  # === return_colour_for_nucleotides
  # ========================================================================= #
  def return_colour_for_nucleotides
    _ = ::Colours.send(USE_THIS_COLOUR_FOR_DNA)
    remove_trailing_escape_part(_)
  end

  # ========================================================================= #
  # === disable_colours
  #
  # If colours-support should be disabled then this method ought to
  # be called.
  # ========================================================================= #
  def disable_colours
    @use_colours = false
  end

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    if @use_colours
      ::Bioroebe.rev # Pointer towards Bioroebe.rev
    else
      ''
    end
  end; alias rev? rev # === rev?

  # ========================================================================= #
  # === erev
  # ========================================================================= #
  def erev(i = '')
    if @use_colours
      ::Bioroebe.erev(i)
    else
      e i
    end
  end

  # ========================================================================= #
  # === remove_trailing_escape_part
  #
  # This method will remove the "closing tag" from Colour-escape codes.
  # ========================================================================= #
  def remove_trailing_escape_part(i)
    ::Colours.remove_trailing_end_from(i)
  end; alias remove_trailing_escape_code remove_trailing_escape_part # === remove_trailing_escape_code
       alias remove_escape_sequence      remove_trailing_escape_part # === remove_escape_sequence
       alias remove_escape_sequences     remove_trailing_escape_part # === remove_escape_sequences

  # ========================================================================= #
  # === colourize_this_nucleotide_sequence
  #
  # This method will colourize the given input-sequence.
  #
  # Note that for strings that contain escape-codes it is better to
  # first sanitize these strings, and remove these escape codes,
  # before proceeding with this method here.
  # ========================================================================= #
  def colourize_this_nucleotide_sequence(
      i, use_colours = @use_colours
    )
    if use_colours
      return return_colour_for_nucleotides+
             i+
             return_colour_for_nucleotides
    end
    return i
  end; alias colourize_dna colourize_this_nucleotide_sequence # === colourize_dna

  # ========================================================================= #
  # === sdir
  # ========================================================================= #
  def sdir(i = '')
    return Colours.sdir(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    return powderblue(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i = '')
    return ::Bioroebe.simp(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end; alias simportant simp # === simportant

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i = '')
    return Colours.swarn(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === eparse
  # ========================================================================= #
  def eparse(i)
    return ::Colours.eparse(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i)
    return ::Bioroebe.sfile(i) if use_colours? and use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === ecomment
  # ========================================================================= #
  def ecomment(i)
    return ::Colours.ecomment(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === red
  # ========================================================================= #
  def red(i)
    return swarn(i) if use_colours_within_the_bioroebe_namespace?
    return i
  end

  # ========================================================================= #
  # === enable_colours
  # ========================================================================= #
  def enable_colours
    @use_colours = true
  end

end; end