#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/base/misc.rb'
# =========================================================================== #
module Bioroebe

class Base

  require 'bioroebe/toplevel_methods/file_and_directory_related_actions.rb'

  # ========================================================================= #
  # === is_palindrome?
  # ========================================================================= #
  def is_palindrome?(i)
    unless ::Bioroebe.respond_to?(:is_palindrome?)
      require 'bioroebe/toplevel_methods/palindromes.rb'
    end
    ::Bioroebe.is_palindrome?(i)
  end; alias is_this_sequence_a_palindrome? is_palindrome? # === is_this_sequence_a_palindrome?

  require 'bioroebe/toplevel_methods/convert_global_env.rb'
  # ========================================================================= #
  # === convert_global_env
  # ========================================================================= #
  def convert_global_env(i)
    ::Bioroebe.convert_global_env(i)
  end

  # ========================================================================= #
  # === register_sigint
  # ========================================================================= #
  def register_sigint(optional_display_this_message = nil)
    Signal.trap('SIGINT') {
      if optional_display_this_message
        e optional_display_this_message
      end
      exit
    }
  end

  # ========================================================================= #
  # === can_base_pair?
  #
  # This method will determine whether two nucleotides can base pair
  # to one another, e. g. A=T or A=U and so forth.
  #
  # A boolean value will be returned.
  # ========================================================================= #
  def can_base_pair?(a, b)
    _ = "#{a}#{b}" 
    case _
    when 'AT','TA','GC','CG','AU','UA'
      return true
    else
      return false
    end
  end

  # ========================================================================= #
  # === directory_to_the_codon_tables?
  #
  # This method will return the directory to the codon-tables, a directory
  # containing the individual .yml files.
  # ========================================================================= #
  def directory_to_the_codon_tables?
    "#{::Bioroebe.project_yaml_directory?}codon_tables/"
  end

  # ========================================================================= #
  # === write_what_into
  #
  # This method can be used to write into a (local) file.
  # ========================================================================= #
  def write_what_into(what, into)
    ::Bioroebe.write_what_into(what, into)
  end; alias save_file write_what_into # === save_file

  # ========================================================================= #
  # === append_what_into
  # ========================================================================= #
  def append_what_into(what, into)
    ::Bioroebe.append_what_into(what, into)
  end

end; end