#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/base/prototype/misc.rb'
# =========================================================================== #
module Bioroebe

class Base

  require 'bioroebe/requires/require_encoding.rb'
  require 'bioroebe/toplevel_methods/file_and_directory_related_actions.rb'
  require 'bioroebe/toplevel_methods/rds.rb'

  require 'bioroebe/toplevel_methods/is_on_roebe.rb'
  # ========================================================================= #
  # === is_on_roebe?
  # ========================================================================= #
  def is_on_roebe?
    ::Bioroebe.is_on_roebe?
  end

  # ========================================================================= #
  # === main_encoding?
  #
  # The main encoding in use. Currently this is hardcoded to UTF-8.
  # ========================================================================= #
  def main_encoding?
    USE_THIS_ENCODING
  end

  # ========================================================================= #
  # === move_file
  #
  # This method can be used to relocate or rename a file.
  # ========================================================================= #
  def move_file(
      this_file, target
    )
    if File.exist? this_file
      begin
        FileUtils.mv(this_file, target)
      rescue Exception => error
        pp error
      end
    else
      e rev+'No file exists at `'+sfile(this_file)+'`.'
    end
  end; alias rename move_file # === rename

  # ========================================================================= #
  # === rds
  #
  # The abbreviation rds stands short for "remove double slashes".
  # ========================================================================= #
  def rds(i)
    ::Bioroebe.rds(i)
  end

  # ========================================================================= #
  # === return_pwd
  # ========================================================================= #
  def return_pwd
    "#{Dir.pwd}/".squeeze('/')
  end; alias return_working_directory return_pwd # === return_working_directory

  # ========================================================================= #
  # === mv
  #
  # Move/Rename a file or directory.
  # ========================================================================= #    
  def mv(old, new)
    FileUtils.mv(old, new)
  end

  # ========================================================================= #
  # === delete_file
  # ========================================================================= #
  def delete_file(i)
    File.delete(i) if File.exist? i
  end; alias remove_file delete_file # === remove_file
       alias remove      delete_file # === remove

  # ========================================================================= #
  # === no_newlines
  #
  # This method will remove all newlines from a given (input) String.
  # ========================================================================= #
  def no_newlines(i)
    i.delete!(N) if i
    return i
  end

  # ========================================================================= #
  # === no_file_exists_at
  # ========================================================================= #
  def no_file_exists_at(i)
    ::Bioroebe.no_file_exists_at(i)
  end

  # ========================================================================= #
  # === file_readlines
  # ========================================================================= #
  def file_readlines(i)
    File.readlines(i, encoding: main_encoding?)
  end; alias readlines file_readlines # === readlines

end; end