#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Color codings for buried amino acids
#
# Will subclass from Score.
# =========================================================================== #
# require 'bioroebe/colours/buried.rb'
# =========================================================================== #
require 'bioroebe/colours/colour_schemes/score.rb'

module Bioroebe::ColourScheme

class Buried < Score # === Bioroebe::ColourScheme::Buried

  # ======================================================================= #
  # === @colours
  # ======================================================================= #
  @colours = {}
  @colours.default = 'FFFFFF' # return white by default

  # ======================================================================= #
  # === Bioroebe::ColourScheme::Buried.score_to_rgb_hex
  # ======================================================================= #
  def self.score_to_rgb_hex(score, min, max)
    percent = score_to_percent(score, min, max)
    rgb_percent_to_hex(0.0, 1.0 - percent, percent)
  end

  # ======================================================================= #
  # === @scores
  # ======================================================================= #
  @scores = { # Hash, with aminoacid->score entries.
    'A' => 0.66,
    'C' => 1.19,
    'D' => 1.46,
    'E' => 0.74,
    'F' => 0.6,
    'G' => 1.56,
    'H' => 0.95,
    'I' => 0.47,
    'K' => 1.01,
    'L' => 0.59,
    'M' => 0.6,
    'N' => 1.56,
    'P' => 1.52,
    'Q' => 0.98,
    'R' => 0.95,
    'S' => 1.43,
    'T' => 0.96,
    'U' => 0,
    'V' => 0.5,
    'W' => 0.96,
    'Y' => 1.14,

    'B' => 1.51,
    'X' => 1.0,
    'Z' => 0.86
  }
  @min = 0.05
  @max = 4.6
  @scores.each { |k,s|
    # ======================================================================= #
    # === Fill up the @colours Hash
    # ======================================================================= #
    @colours[k] = score_to_rgb_hex(s, @min, @max) # Convert the scores into RGB values.
  }

end; end