#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Bioroebe::ColourScheme::Taylor
#
# Colour codings for Taylor, for amino acids.
#
# These colours were defined by Willie Taylor. An entertaining
# description of their birth can be found in Protein Engineering,
# Vol 10 , 743-746 (1997)
#
#   See here: https://peds.oxfordjournals.org/content/10/7/743.full.pdf
#
# A html-page showing these results can be found here:
#
#   https://www.jalview.org/help/html/colourSchemes/taylor.html
#
# =========================================================================== #
require 'bioroebe/colours/colour_schemes/simple.rb'

module Bioroebe::ColourScheme

class Taylor < Simple # === Bioroebe::ColourScheme::Taylor

  # ========================================================================= #
  # === @colours
  # ========================================================================= #
  @colours = {
    'A' => 'CCFF00',
    'C' => 'FFFF00',
    'D' => 'FF0000',
    'E' => 'FF0066',
    'F' => '00FF66',
    'G' => 'FF9900',
    'H' => '0066FF',
    'I' => '66FF00',
    'K' => '6600FF',
    'L' => '33FF00',
    'M' => '00FF00',
    'N' => 'CC00FF',
    'P' => 'FFCC00',
    'Q' => 'FF00CC',
    'R' => '0000FF',
    'S' => 'FF3300',
    'T' => 'FF6600',
    'U' => 'FFFFFF',
    'V' => '99FF00',
    'W' => '00CCFF',
    'Y' => '00FFCC',

    'B' => 'FFFFFF',
    'X' => 'FFFFFF',
    'Z' => 'FFFFFF'
  }
  @colours.default = 'FFFFFF' # Use white by default.

end; end