#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Bioroebe::ColourScheme::Zappo
#
# Colour codings for Zappo, for amino acids.
#
# The colour code for this scheme is as follows:
#
#   Aromatic    aminoacids (FYW)  will be: #FFC800
#   Positive    aminoacids (KRH)  will be: #6464FF
#   Negative    aminoacids (DE)   will be: #FF0000
#   Hydrophilic aminoacids (STNQ) will be: 
#   Conformationally special aminoacids (PG)
#   Cysteine                  (C) will be:
#
# =========================================================================== #
# Zappo depends on class Bioroebe::ColourScheme::Simple.
# ====================Bioroebe::ColourSchemeDemo======================================================= #
require 'bioroebe/colours/colour_schemes/simple.rb'

module Bioroebe::ColourScheme

class Zappo < Simple # === Bioroebe::ColourScheme::Zappo

  # ========================================================================= #
  # === @colours
  # ========================================================================= #
  @colours = {
    'A' => 'FFAFAF',
    'C' => 'FFFF00',
    'D' => 'FF0000',
    'E' => 'FF0000',
    'F' => 'FFC800',
    'G' => 'FF00FF',
    'H' => 'FF0000',
    'I' => 'FFAFAF',
    'K' => '6464FF',
    'L' => 'FFAFAF',
    'M' => 'FFAFAF',
    'N' => '00FF00',
    'P' => 'FF00FF',
    'Q' => '00FF00',
    'R' => '6464FF',
    'S' => '00FF00',
    'T' => '00FF00',
    'U' => 'FFFFFF',
    'V' => 'FFAFAF',
    'W' => 'FFC800',
    'Y' => 'FFC800',

    'B' => 'FFFFFF',
    'X' => 'FFFFFF',
    'Z' => 'FFFFFF',
  }
  @colours.default = 'FFFFFF' # return white by default

end; end