#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file will contain support for colours, through the 'colours' gem.
#
# The reason why this resides here is that we need a way to also modify
# the colours for the whole Bioroebe project, that is, the main namespace
# called Bioroebe. This is why there are topleve-level instance variables
# such as @use_colours. Setting this variable to false will disable
# colours for the Bioroebe project.
# =========================================================================== #
# require 'bioroebe/colours/colours.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/requires/require_colours.rb'
  require 'bioroebe/colours/misc_colours.rb'
  require 'bioroebe/colours/rev.rb'
  require 'bioroebe/colours/sdir.rb'
  require 'bioroebe/colours/sfancy.rb'
  require 'bioroebe/colours/sfile.rb'
  require 'bioroebe/colours/simp.rb'
  require 'bioroebe/colours/swarn.rb'

  # ========================================================================= #
  # === Bioroebe::USE_THIS_COLOUR_FOR_DNA
  #
  # The following constant will denote which colour we will use for DNA
  # sequences by default, in this case, the HTML colour called steelblue.
  # ========================================================================= #
  USE_THIS_COLOUR_FOR_DNA = :steelblue

  # ========================================================================= #
  # === @default_colour
  #
  # This colour is used by default.
  # ========================================================================= #
  @default_colour = nil
  @default_colour = Colours::GREEN if @use_colours

  # ========================================================================= #
  # === Bioroebe.default_colour?
  # ========================================================================= #
  def self.default_colour?
    @default_colour
  end; self.instance_eval { alias main_colour default_colour? } # === Bioroebe.main_colour
       self.instance_eval { alias main_col    default_colour? } # === Bioroebe.main_col

  # ========================================================================= #
  # === default_colour?
  # ========================================================================= #  
  def default_colour?
    ::Bioroebe.default_colour?
  end; alias main_colour default_colour? # === main_colour
       alias main_col    default_colour? # === main_col

  # ========================================================================= #
  # === Bioroebe.set_default_colour
  #
  # This is primarily used by the Bioroebe::Shell.
  # ========================================================================= # 
  def self.set_default_colour(i)
    @default_colour = i
  end

  # ========================================================================= #
  # === use_colours_in_general?
  # ========================================================================= #
  def use_colours_in_general?
    ::Bioroebe.use_colours?
  end

  # ========================================================================= #
  # === Bioroebe.enable_colours
  #
  # Use this method to enable colours for the whole Bioroebe project.
  #
  # All classes that are part of the Bioroebe project should honour
  # this setting (if it is a class that may make use of colours;
  # some smaller classes do not need colours, and hence have no need
  # for the method here).
  # ========================================================================= #
  def self.enable_colours
    @use_colours = true
  end; self.instance_eval { alias enablecolours enable_colours } # === Bioroebe.enablecolours

  # ========================================================================= #
  # === Bioroebe.colours
  #
  # This method can be used to quickly enable or disable colours,
  # by passing '+' or '-'.
  # ========================================================================= #
  def self.colours(enable_or_disable = '+')
    case enable_or_disable.to_s
    when '+',
         'true'
      enable_colours
    when '-',
         'false',
         ''
      disable_colours
    end
  end

  # ========================================================================= #
  # === Bioroebe.disable_colours
  #
  # Use this method if you wish to disable colours for the whole Bioroebe
  # project.
  # ========================================================================= #
  def self.disable_colours(be_verbose = false)
    if be_verbose
      e 'Disabling colours.'
    end
    @use_colours = false
  end

end