#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Do note that in the past, this .rb file contained several hardcoded
# constants. Since as of September 2019 this older approach was deprecated,
# mostly because hardcoded constants do not allow for as much flexibility
# as method calls do. Even if a method call is slightly slower, I found
# that in the long run it greatly makes the resulting code significantly
# simpler to maintain, rather than having to deal with hardcoded constants
# that may lead to inflexibility. This happened to me in a restricted
# university setting, so I don't quite fancy hardcoded constants anymore
# as much - at the least not for important variables.
# =========================================================================== #
# require 'bioroebe/constants/base_directory.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/project/project.rb'
  require 'bioroebe/toplevel_methods/is_on_roebe.rb'

  # ========================================================================= #
  # === Bioroebe.base_directory?
  #
  # This method will return the stored value for the @base_directory,
  # which is typically a String such as "/root/Bioroebe/".
  # ========================================================================= #
  def self.base_directory?
    @base_directory
  end; self.instance_eval { alias base_dir? base_directory? } # === Bioroebe.base_dir?

  # ========================================================================= #
  # === Bioroebe.set_base_directory
  #
  # This method can be used to set the main @base_dir variable
  # for the Bioroebe-project.
  #
  # By default this method will append the name "Bioroebe/" to the
  # target directory, if :default is given as argument - which is
  # also the default argument, hence the name. If you do not want
  # this behaviour then you have to call Bioroebe.set_base_directory()
  # by yourself, with the desired base directory to use.
  # ========================================================================= #
  def self.set_base_directory(
      i = :default
    )
    # ======================================================================= #
    # We may need YAML support in this method.
    # ======================================================================= #
    require 'bioroebe/requires/require_yaml.rb'
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      user_home_directory = user_home_directory?
      unless File.directory? user_home_directory
        # =================================================================== #
        # Enter this clause for regular systems that do not have a
        # /Depot/Temp/ directory. Such systems will store output
        # into the home directory of the user.
        # =================================================================== #
        i = "#{File.expand_path('~')}/"
      end
      file_temp_dir = "#{Bioroebe.project_yaml_directory?}configuration/temp_dir.yml"
      if File.exist?(file_temp_dir) and !File.empty?(file_temp_dir)
        i = YAML.load_file(file_temp_dir)
      else
        i = "#{user_home_directory}bioroebe/"
      end
      # ===================================================================== #
      # Modify the path a little on my home setup.
      # ===================================================================== #
      if is_on_roebe?
        i << 'bioroebe/' unless i.include? 'bioroebe'
      end
    end
    # ======================================================================= #
    # Keep a trailing '/' for the variable at all times.
    # ======================================================================= #
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    @base_directory = i
  end; self.instance_eval { alias set_base_dir set_base_directory } # === Bioroebe.set_base_dir

  # ========================================================================= #
  # === Bioroebe.user_home_directory?
  #
  # This method will either refer to the home directory of the user at hand,
  # or, if unavailable, simply refer to '/tmp/' as hardcoded target.
  #
  # The main assumption here is that the home directory is writable and
  # readable for the user, by default.
  # ========================================================================= #
  def self.user_home_directory?
    begin
      # ===================================================================== #
      # Expand to find the proper directory.
      # ===================================================================== #
      return "#{File.expand_path('~')}/"
    rescue ArgumentError
      # ===================================================================== #
      # Rescued in .cgi environments, where it will default to /tmp/.
      # ===================================================================== #
      return '/tmp/'
    end
  end

  set_base_directory # Initialize it at once.

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e 'Base directory for the Bioroebe project: '+
    Bioroebe.base_directory?
  e Bioroebe.user_home_directory?
end