#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/constants/codon_tables.rb'
# =========================================================================== #
module Bioroebe

module CodonTablesFrequencies

  require 'bioroebe/project/project.rb'
  require 'bioroebe/constants/files_and_directories.rb'

  # ========================================================================= #
  # === FILE_HOMO_SAPIENS
  #
  # Note that before the rewrite in September 2019, there was another
  # old constant referring to this entry here, called
  # CODON_TABLE_FOR_HOMO_SAPIENS. That latter constant was removed in
  # 2019, to simplify the code and make it overall more consistent.
  # ========================================================================= #
  FILE_HOMO_SAPIENS =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "9606_Homo_sapiens.yml"

  # ========================================================================= #
  # === FILE_E_COLI
  #
  # This is for the bacterium E. coli.
  # ========================================================================= #
  FILE_E_COLI =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "83333_Escherichia_coli_K12.yml"

  # ========================================================================= #
  # === FILE_SACCHAROMYCES_CEREVISIAE
  #
  # This is for the yeast Saccharomyces cerevisiae.
  # ========================================================================= #
  FILE_SACCHAROMYCES_CEREVISIAE =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "4932_Saccharomyces_cerevisiae.yml"

  # ========================================================================= #
  # === FILE_FELIS_CATUS
  #
  # This is for the domestic cat.
  # ========================================================================= #
  FILE_FELIS_CATUS =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "9685_Felis_catus.yml"

  # ========================================================================= #
  # === FILE_ARABIDOPSIS_THALIANA
  #
  # This is for Arabidopsis thaliana.
  # ========================================================================= #
  FILE_ARABIDOPSIS_THALIANA =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "3702_Arabidopsis_thaliana.yml"

  # ========================================================================= #
  # === FILE_DROSOPHILA_MELANOGASTER
  #
  # This is for the fruit fly.
  # ========================================================================= #
  FILE_DROSOPHILA_MELANOGASTER =
    "#{::Bioroebe::DIRECTORY_CODON_TABLES_FREQUENCIES}"\
    "7227_Drosophila_melanogaster.yml"

end; end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe::CodonTablesFrequencies::FILE_DROSOPHILA_MELANOGASTER  
end