#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Bioroebe::Postgresql
#
# This file stores any required information to login into a postgresql
# database. This can be on your home system or as part of a cluster.
#
# The below command is equivalent to the following command-line:
#
#     psql --host=bunuel.imp.univie.ac.at --port=5471 --username=robert --dbname=robert_db --quiet
#
# =========================================================================== #
# POSTGRE_LOGIN_COMMAND = '
#   psql --host=bunuel.imp.univie.ac.at --port=5471 --username=robert --dbname=robert_db --quiet
# '.delete("\n").strip
# =========================================================================== #
# require 'bioroebe/constants/database_constants.rb'
# =========================================================================== #
module Bioroebe

module Postgresql # === Bioroebe::Postgresql

  # ========================================================================= #
  # === Bioroebe::Postgresql
  #
  # The following constants can be modified to account for different
  # password, username, dbname and so forth.
  # ========================================================================= #

  # ========================================================================= #
  # === POSTGRE_HOST
  # ========================================================================= #
  POSTGRE_HOST        = 'bunuel.imp.univie.ac.at'

  # ========================================================================= #
  # === POSTGRE_PORT
  # ========================================================================= #
  POSTGRE_PORT        = '5471'

  # ========================================================================= #
  # === POSTGRE_USERNAME
  # ========================================================================= #
  POSTGRE_USERNAME    = 'robert'

  # ========================================================================= #
  # === POSTGRE_DBNAME
  # ========================================================================= #
  POSTGRE_DBNAME      = 'robert_db'

  # ========================================================================= #
  # === POSTGRE_EXTRA_FLAGS  
  # ========================================================================= #
  POSTGRE_EXTRA_FLAGS = '--quiet'

  # ========================================================================= #
  # === POSTGRE_PASSWORD
  # ========================================================================= #
  POSTGRE_PASSWORD    = 'robert123'

  # ========================================================================= #
  # === POSTGRE_LOGIN_COMMAND
  #
  # This constant will become the full postgre login command.
  # ========================================================================= #
  POSTGRE_LOGIN_COMMAND = ('
    psql --host='+POSTGRE_HOST+' --port='+POSTGRE_PORT+
    ' --username='+POSTGRE_USERNAME+
    ' --dbname='+POSTGRE_DBNAME+
    ' '+POSTGRE_EXTRA_FLAGS+'
  '.delete("\n").strip).strip
  POSTGRE_LOGIN_COMMAND_HOME = POSTGRE_LOGIN_COMMAND

  # ========================================================================= #
  # === POSTGRE_LOGIN_COMMAND_NO_HEADERS
  #
  # Here we use --tuples-only.
  # ========================================================================= #
  POSTGRE_LOGIN_COMMAND_NO_HEADERS = POSTGRE_LOGIN_COMMAND+' 
    --tuples-only  
  '.delete("\n").squeeze(' ') # --quiet

  # ========================================================================= #
  # === POSTGRE_DROP_NODES_COMMAND
  # ========================================================================= #
  POSTGRE_DROP_NODES_COMMAND = DROP_TABLE_NODES = 'DROP TABLE nodes;'
  POSTGRE_DROP_NAMES_COMMAND = DROP_TABLE_NAMES = 'DROP TABLE names;'
  DROP_TABLE_FASTA = 'DROP TABLE fasta;'

  # ========================================================================= #
  # === POSTGRE_DROP_FASTA_COMMAND
  # ========================================================================= #
  POSTGRE_DROP_FASTA_COMMAND = DROP_FASTA_TABLE = DROP_TABLE_FASTA

end; end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  include Bioroebe::Postgresql
  e POSTGRE_LOGIN_COMMAND
  e POSTGRE_LOGIN_COMMAND_HOME
  e POSTGRE_LOGIN_COMMAND_NO_HEADERS
  e POSTGRE_DROP_NODES_COMMAND
  e POSTGRE_DROP_NAMES_COMMAND
  e POSTGRE_DROP_FASTA_COMMAND
end # rb database_constants.rb