#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/constants/files_and_directories.rb'
# Bioroebe.load_and_return_the_restriction_enzymes
# Bioroebe.blosum_directory?
# =========================================================================== #
module Bioroebe

  require 'bioroebe/project/project.rb'
  require 'bioroebe/requires/require_yaml.rb'

  # ========================================================================= #
  # === FILE_HYDROPATHY_TABLE
  # ========================================================================= #
  FILE_HYDROPATHY_TABLE =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}hydropathy_table.yml"

  # ========================================================================= #
  # === FILE_NUCLEAR_LOCALIZATION_SEQUENCES
  # ========================================================================= #
  FILE_NUCLEAR_LOCALIZATION_SEQUENCES =
    "#{project_yaml_directory?}nuclear_localization_sequences.yml"

  # ========================================================================= #
  # === FILE_DEFAULT_COLOURS_FOR_THE_AMINOACIDS
  # ========================================================================= #
  FILE_DEFAULT_COLOURS_FOR_THE_AMINOACIDS =
    "#{project_yaml_directory?}configuration/default_colours_for_the_aminoacids.yml"

  # ========================================================================= #
  # === FILE_BROWSER
  # ========================================================================= #
  FILE_BROWSER =
    "#{project_yaml_directory?}configuration/browser.yml"

  # ========================================================================= #
  # === FILE_AMINOACIDS_MOLECULAR_FORMULA
  # ========================================================================= #
  FILE_AMINOACIDS_MOLECULAR_FORMULA =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_molecular_formula.yml"

  # ========================================================================= #
  # === FILE_AMINOACIDS_THREE_TO_ONE
  # ========================================================================= #
  FILE_AMINOACIDS_THREE_TO_ONE = 
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_three_to_one.yml"

  # ========================================================================= #
  # === FILE_WEIGHT_OF_COMMON_PROTEINS
  # ========================================================================= #
  FILE_WEIGHT_OF_COMMON_PROTEINS = 
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}weight_of_common_proteins.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_LONG_NAME_TO_ONE_LETTER
  # ========================================================================= #
  FILE_AMINO_ACIDS_LONG_NAME_TO_ONE_LETTER =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_long_name_to_one_letter.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_MOLECULAR_FORMULA
  # ========================================================================= #
  FILE_AMINO_ACIDS_MOLECULAR_FORMULA =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_molecular_formula.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_MASS_TABLE
  #
  #   bl $BIOROEBE_YAML/aminoacids/amino_acids_monoisotopic_mass_table.yml
  #
  # ========================================================================= #
  FILE_AMINO_ACIDS_MASS_TABLE =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_monoisotopic_mass_table.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS
  # ========================================================================= #
  FILE_AMINO_ACIDS =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_ABBREVIATIONS
  # ========================================================================= #
  FILE_AMINO_ACIDS_ABBREVIATIONS =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_abbreviations.yml"

  # ========================================================================= #
  # === Bioroebe.file_amino_acids_abbreviations
  # ========================================================================= #
  def self.file_amino_acids_abbreviations
    FILE_AMINO_ACIDS_ABBREVIATIONS
  end

  # ========================================================================= #
  # === AMINO_ACIDS_ABBREVIATIONS
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_ABBREVIATIONS
    begin
      # $BIOROEBE/amino_acids_abbreviations.yml
      AMINO_ACIDS_ABBREVIATIONS = YAML.load_file(FILE_AMINO_ACIDS_ABBREVIATIONS)
    rescue Psych::SyntaxError => error
      pp error
      AMINO_ACIDS_ABBREVIATIONS = {}
    end
  end

  # ========================================================================= #
  # === DIRECTORY_CODON_TABLES_FREQUENCIES
  #
  # This constant may point to a directory such as:
  #
  #   /home/Programs/Ruby/2.7.0/lib/ruby/site_ruby/2.7.0/bioroebe/codon_tables/frequencies/
  #
  # ========================================================================= #
  DIRECTORY_CODON_TABLES_FREQUENCIES =
    "#{CODON_TABLES_DIRECTORY}frequencies/"

  # ========================================================================= #
  # === FILE_NUCLEOTIDES
  # ========================================================================= #
  FILE_NUCLEOTIDES =
    "#{project_yaml_directory?}nucleotides/nucleotides.yml"

  # ========================================================================= #
  # === NUCLEOTIDES
  # ========================================================================= #
  if File.exist? FILE_NUCLEOTIDES
    NUCLEOTIDES = YAML.load_file(FILE_NUCLEOTIDES)
  else
    NUCLEOTIDES = nil
  end

  require 'bioroebe/toplevel_methods/log_directory.rb'
  # ========================================================================= #
  # === Bioroebe.file_statistics?
  #
  # This file can normally be found here:
  #
  #   $BIOROEBE/yaml/statistics.yml
  #
  # ========================================================================= #
  def self.file_statistics?
    "#{Bioroebe.log_dir?}statistics.yml"
  end

  # ========================================================================= #
  # === FILE_FASTQ_QUALITY_SCHEMES
  #
  # This constant will point to a location such as this one here:
  #
  #   /Programs/Ruby/2.6.4/lib/ruby/site_ruby/2.6.0/bioroebe/yaml/fastq_quality_schemes.yml
  #
  # ========================================================================= #
  FILE_FASTQ_QUALITY_SCHEMES =
    "#{project_yaml_directory?}fastq_quality_schemes.yml"

  # ========================================================================= #
  # === Bioroebe.file_talens
  # ========================================================================= #
  def self.file_talens
    "#{project_yaml_directory?}talens.yml"
  end

  # ========================================================================= #
  # === FILE_GFP_SEQUENCE
  # ========================================================================= #
  FILE_GFP_SEQUENCE = 
    "#{project_yaml_directory?}sequences/"\
    "JX472995_Green_fluorescent_protein_from_Aequorea_victoria.fasta"

  # ========================================================================= #
  # === FILE_RESTRICTION_ENZYMES
  #
  # bl $BIOROEBE/yaml/restriction/enzymes/restriction_enzymes.yml
  # ========================================================================= #
  FILE_RESTRICTION_ENZYMES =
    "#{project_yaml_directory?}restriction_enzymes/restriction_enzymes.yml"

  # ========================================================================= #
  # === FILE_COLOURIZE_FASTA_SEQUENCES
  #
  # This constants points to the .yml file that will hold information
  # in how to colourize the FASTA sequences.
  # ========================================================================= #
  FILE_COLOURIZE_FASTA_SEQUENCES =
    "#{project_yaml_directory?}configuration/colourize_fasta_sequences.yml"

  # ========================================================================= #
  # === FILE_BLOSUM45
  # ========================================================================= #
  FILE_BLOSUM45 = "#{BLOSUM_DIRECTORY}/blosum45.yml"

  # ========================================================================= #
  # === FILE_BLOSUM50
  # ========================================================================= #
  FILE_BLOSUM50 = "#{BLOSUM_DIRECTORY}/blosum50.yml"

  # ========================================================================= #
  # === FILE_BLOSUM62
  # ========================================================================= #
  FILE_BLOSUM62 = "#{BLOSUM_DIRECTORY}/blosum62.yml"

  # ========================================================================= #
  # === FILE_BLOSUM80
  # ========================================================================= #
  FILE_BLOSUM80 = "#{BLOSUM_DIRECTORY}/blosum80.yml"

  # ========================================================================= #
  # === FILE_BLOSUM90
  # ========================================================================= #
  FILE_BLOSUM90 = "#{BLOSUM_DIRECTORY}/blosum90.yml"

  # ========================================================================= #
  # === BLOSUM_MATRIX
  # ========================================================================= #
  FILE_BLOSUM_MATRIX = "#{BLOSUM_DIRECTORY}blosum_matrix.yml"

  # ========================================================================= #
  # === Bioroebe.blosum_matrix
  # ========================================================================= #
  def self.blosum_matrix(i = FILE_BLOSUM_MATRIX)
    YAML.load_file(i)
  end

  # ========================================================================= #
  # === HYDROPATHY_TABLE
  # ========================================================================= #
  if File.exist? FILE_HYDROPATHY_TABLE
    HYDROPATHY_TABLE =
      YAML.load_file(
        FILE_HYDROPATHY_TABLE
      )
  end

  # ========================================================================= #
  # === FILE_CHROMOSOME_NUMBERS
  # ========================================================================= #
  FILE_CHROMOSOME_NUMBERS =
    "#{project_yaml_directory?}chromosomes/chromosome_numbers.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_FREQUENCY
  # ========================================================================= #
  FILE_AMINO_ACIDS_FREQUENCY =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_frequency.yml"

  # ========================================================================= #
  # === Bioroebe.file_amino_acids_frequency
  # ========================================================================= #
  def self.file_amino_acids_frequency
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_frequency.yml"
  end

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_RESTE_YAML
  # ========================================================================= #
  FILE_AMINO_ACIDS_RESTE_YAML =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_reste.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_THREE_TO_ONE
  #
  # We'll keep the keys downcased.
  #
  #   bl $RUBY_SRC/bioroebe/lib/bioroebe/yaml/aminoacids/amino_acids_three_to_one.yml
  #
  # ========================================================================= #
  FILE_AMINO_ACIDS_THREE_TO_ONE =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_three_to_one.yml"

  # ========================================================================= #
  # === FILE_AMINO_ACIDS_AVERAGE_MASS_TABLE
  #
  # This will point to the file amino_acids_average_mass_table.yml.
  # ========================================================================= #
  FILE_AMINO_ACIDS_AVERAGE_MASS_TABLE =
    "#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_average_mass_table.yml"

  # ========================================================================= #
  # === FILE_NUCLEOTIDES_WEIGHT
  #
  # The path to the file that holds the weight of the nucleotides.
  # ========================================================================= #
  FILE_NUCLEOTIDES_WEIGHT =
    "#{project_yaml_directory?}nucleotides/nucleotides_weight.yml"

  # ========================================================================= #
  # === Bioroebe.load_and_return_the_restriction_enzymes
  #
  # This method will load, and then return the restriction enzymes that
  # are bundled within the bioroebe-project.
  # ========================================================================= #
  def self.load_and_return_the_restriction_enzymes
    YAML.load_file(FILE_RESTRICTION_ENZYMES)
  end

  # ========================================================================= #
  # === Bioroebe.file_restriction_enzymes
  # ========================================================================= #
  def file_restriction_enzymes
    FILE_RESTRICTION_ENZYMES
  end

  # ========================================================================= #
  # === Bioroebe.blosum_directory?
  # ========================================================================= #
  def self.blosum_directory?
    "#{project_yaml_directory?}blosum/"
  end

  # ========================================================================= #
  # === AMINO_ACIDS_MOLECULAR_FORMULA
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_MOLECULAR_FORMULA
    AMINO_ACIDS_MOLECULAR_FORMULA = YAML.load_file(
      FILE_AMINO_ACIDS_MOLECULAR_FORMULA
    )
  end

  # ========================================================================= #
  # === AMINO_ACIDS_RESTE
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_RESTE_YAML
    hash = {}
    begin
      _ = YAML.load_file(FILE_AMINO_ACIDS_RESTE_YAML)
      # ======================================================================= #
      # As of August 2015, we will keep the keys downcased.
      # ======================================================================= #
      _.each_pair {|key, value|
        key = key.dup if key.frozen?
        key.downcase!
        hash[key] = value
      }
      AMINO_ACIDS_RESTE = hash
    rescue Psych::SyntaxError
      AMINO_ACIDS_RESTE = {}
    end
  else
    AMINO_ACIDS_RESTE = {}
  end

  # ========================================================================= #
  # === AMINO_ACIDS_LONG_NAME_TO_ONE_LETTER
  #
  # Load up the dataset stored in the file
  # amino_acids_long_name_to_one_letter.yml.
  # ========================================================================= #
  _ = FILE_AMINO_ACIDS_LONG_NAME_TO_ONE_LETTER
  if File.exist? _
    AMINO_ACIDS_LONG_NAME_TO_ONE_LETTER = YAML.load_file(_)
  else
    puts "No file exists at #{_}."
  end

  # ========================================================================= #
  # === AMINO_ACIDS_MASS_TABLE
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_MASS_TABLE # bl $BIOROEBE/yaml/amino_acids_mass_table.yml
    AMINO_ACIDS_MASS_TABLE = YAML.load_file(FILE_AMINO_ACIDS_MASS_TABLE)
  else # Else hardcode the AminoAcid table here. This may no longer be necessary, though.
    AMINO_ACIDS_MASS_TABLE = {
      'A' =>  71.03711, 'C' => 103.00919, 'D' => 115.02694,
      'E' => 129.04259, 'F' => 147.06841, 'G' =>  57.02146,
      'H' => 137.05891, 'I' => 113.08406, 'K' => 128.09496,
      'L' => 113.08406, 'M' => 131.04049, 'N' => 114.04293,
      'P' =>  97.05276, 'Q' => 128.05858, 'R' => 156.10111,
      'S' =>  87.03203, 'T' => 101.04768, 'V' =>  99.06841,
      'W' => 186.07931, 'Y' => 163.06333
    }
  end; AMINO_ACIDS_AVERAGE_MONOISOTOPIC_TABLE = AMINO_ACIDS_MASS_TABLE # An alias.


  # ========================================================================= #
  # Next determine some important constants.
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS
    # ======================================================================= #
    # === Bioroebe::AMINO_ACIDS
    #
    # Currently listing 21 AminoAcids from amino_acids.yml
    #
    #   bl $BIOROEBE/yaml/aminoacids/amino_acids.yml
    #
    # ======================================================================= #
    AMINO_ACIDS = YAML.load_file(
      FILE_AMINO_ACIDS
    )
    # ======================================================================= #
    # === ::Bioroebe::AMINO_ACIDS_ENGLISH
    # ======================================================================= #
    FILE_AMINO_ACIDS_ENGLISH = YAML.load_file("#{BIOROEBE_YAML_AMINOACIDS_DIRECTORY}amino_acids_english.yml")
      AMINO_ACIDS_ENGLISH = FILE_AMINO_ACIDS_ENGLISH # === AMINO_ACIDS_ENGLISH
  end

  # ========================================================================= #
  # === Bioroebe.aminoacids?
  #
  # Note that this will return a Hash that looks like this:
  #
  #   {"A"=>{"ala"=>"alanine", "d
  #
  # ========================================================================= #
  def self.aminoacids?
    AMINO_ACIDS
  end

  # ========================================================================= #
  # === AMINO_ACIDS_AVERAGE_MASS_TABLE
  #
  # And here is the average mass table.
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_AVERAGE_MASS_TABLE # bl $BIOROEBE/yaml/aminoacids/amino_acids_average_mass_table.yml
    AMINO_ACIDS_AVERAGE_MASS_TABLE = YAML.load_file(FILE_AMINO_ACIDS_AVERAGE_MASS_TABLE)
  else # Else simply hardcode the AminoAcid table here.
    AMINO_ACIDS_AVERAGE_MASS_TABLE = {
      'A' =>  71.0788,
      'C' => 103.1388,
      'D' => 115.0886,
      'E' => 129.1155,
      'F' => 147.1766,
      'G' =>  57.0519,
      'H' => 137.1411,
      'I' => 113.1594,
      'K' => 128.1741,
      'L' => 113.1594,
      'M' => 131.1926,
      'N' => 114.1038,
      'P' =>  97.1167,
      'Q' => 128.1307,
      'R' => 156.1875,
      'S' =>  87.0782,
      'T' => 101.1051,
      'V' =>  99.1326,
      'W' => 186.2132,
      'Y' => 163.1760
    }
  end

  # ========================================================================= #
  # === AMINO_ACIDS_THREE_TO_ONE
  #
  # An "alias" to this constant exists, named AA_THREE_TO_ONE.
  #
  #   bl $BIOROEBE/yaml/aminoacids/amino_acids_three_to_one.yml
  #
  # ========================================================================= #
  if File.exist? FILE_AMINO_ACIDS_THREE_TO_ONE
    hash = {}
    YAML.load_file(
      FILE_AMINO_ACIDS_THREE_TO_ONE
    ).each {|key, value|
      hash[key.to_s.downcase] = value
    }
    AMINO_ACIDS_THREE_TO_ONE = hash
             # AA_THREE_TO_ONE = AMINO_ACIDS_THREE_TO_ONE # Legacy pointer. Is mildly deprecated.
             # ^^^ Disabled as of 25.03.2020. Use the longer variant instead. 
  end

  # ========================================================================= #
  # === NUCLEAR_LOCALIZATION_SEQUENCES
  #
  # We must be careful and check whether it exists or not.
  # ========================================================================= #
  _ = FILE_NUCLEAR_LOCALIZATION_SEQUENCES
  if File.exist? _
    begin
      NUCLEAR_LOCALIZATION_SEQUENCES = YAML.load_file(_)
      # ======================================================================= #
      # === Bioroebe::ARRAY_NLS_SEQUENCES
      #
      # All NLS sequences should be registered here.
      # ======================================================================= #
      ARRAY_NLS_SEQUENCES = NUCLEAR_LOCALIZATION_SEQUENCES.values.map {|entry|
        entry = entry[0, entry.index(' ')] if entry.include? ' '
        entry.delete!('[') if entry.include? '['
        entry.delete!(']') if entry.include? ']'
        entry
      }
    rescue Psych::SyntaxError => error
      pp error
      ARRAY_NLS_SEQUENCES = []
    end
  else
    NUCLEAR_LOCALIZATION_SEQUENCES = ''
    ARRAY_NLS_SEQUENCES = []
  end

  # ========================================================================= #
  # === Bioroebe.random_aminoacid?
  #
  # This method will return a random aminoacid.
  #
  # A number can be passed to this method, which specifies how many
  # random aminoacids are to be returned, e. g. 20 as argument refers
  # to 20 aminoacids that will be generated here.
  #
  # Usage example:
  #
  #   Bioroebe.random_aminoacid? 20 # => "UAVHYQQESWUYAOVESEIY"
  #
  # ========================================================================= #
  def self.random_aminoacid?(optional_return_n_aminoacids = 1)
    case optional_return_n_aminoacids
    when 1
      AMINO_ACIDS.keys.sample # This should be a bit faster.
    else
      optional_return_n_aminoacids.to_i.times.map {
        AMINO_ACIDS.keys.sample
      }.join.strip
    end
  end; self.instance_eval { alias random_aminoacid        random_aminoacid? } # === Bioroebe.random_aminoacid
       self.instance_eval { alias return_random_aminoacid random_aminoacid? } # === Bioroebe.return_random_aminoacid

  # ========================================================================= #
  # === Bioroebe.return_array_of_common_aminoacids
  # ========================================================================= #
  def self.return_array_of_common_aminoacids
    AMINO_ACIDS.keys - ['O'] - ['U']
  end

  # ========================================================================= #
  # === FILE_ALU_ELEMENTS
  # ========================================================================= #
  # FILE_ALU_ELEMENTS =
  #   "#{project_yaml_directory?}alu_elements.yml"

  # ========================================================================= #
  # === Bioroebe.file_molecular_weight
  # ========================================================================= #
  def self.file_molecular_weight
    "#{project_yaml_directory?}aminoacids/molecular_weight.yml"
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  include Bioroebe
  e FILE_FASTQ_QUALITY_SCHEMES
  pp NUCLEOTIDES
  e 'Restriction enzymes can be found here: '+
    FILE_RESTRICTION_ENZYMES
  e FILE_HYDROPATHY_TABLE
  pp HYDROPATHY_TABLE
  e '='*80
  e FILE_AMINO_ACIDS_FREQUENCY
  e FILE_NUCLEOTIDES_WEIGHT
  e FILE_AMINOACIDS_MOLECULAR_FORMULA
  e FILE_AMINOACIDS_THREE_TO_ONE
  e '='*80
  pp AMINO_ACIDS
  e '='*80
  pp AMINO_ACIDS_AVERAGE_MASS_TABLE
  e '='*80
  pp AMINO_ACIDS_THREE_TO_ONE
  e '='*80
end