#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/constants/misc.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/constants/aminoacids_and_proteins.rb'
  # require 'bioroebe/constants/roebe.rb' # <- Not sure if we still need this.

  # ========================================================================= #
  # === array_failsave_requires
  #
  # Next we do some failsafe requires for the Bioroebe::Shell component
  # specifically.
  # ========================================================================= #
  array_failsave_requires = %w(
    convert_global_env
  )
  array_failsave_requires.each {|project|
    begin
      require project
    rescue LoadError; end # Silent rescue.
  }

  # ========================================================================= #
  # === E_BOX
  #
  # This constant was disabled in December 2020. It may have to be moved
  # to .rb files that depend on it, if we still actually need it. If
  # not then this comment will be removed at a later point anyway.
  # ========================================================================= #
  # E_BOX = 'CACGTG' # The "E Box".

  # ========================================================================= #
  # === BE_VERBOSE
  # ========================================================================= #
  BE_VERBOSE = true

  # ========================================================================= #
  # === TOKEN                                                     (TOKEN tag)
  # ========================================================================= #
  TOKEN = '|'

  # ========================================================================= #
  # === NAMES_ENTRIES
  #
  # This used to belong to the Taxonomy submodule.
  # ========================================================================= #
  NAMES_ENTRIES = 'names.sql'

  # ========================================================================= #
  # === NODES_ENTRIES
  #
  # This used to belong to the Taxonomy submodule.
  # ========================================================================= #
  NODES_ENTRIES = 'nodes.sql'

  # ========================================================================= #
  # === FASTA_ENTRIES
  #
  # This used to belong to the Taxonomy submodule.
  # ========================================================================= #
  FASTA_ENTRIES = 'fasta.sql'

  # ========================================================================= #
  # === SHALL_WE_LOG_LAST_UPDATE
  #
  # This constant exists specifically for the taxonomy-component of
  # the Bioroebe project.
  # ========================================================================= #
  SHALL_WE_LOG_LAST_UPDATE = true

  require 'bioroebe/toplevel_methods/log_directory.rb'
  # ========================================================================= #
  # === TAXONOMY_NCBI_DATABASE_LAST_UPDATE_LOG_FILE
  #
  # This constant is used specifically for the taxonomy-component of the
  # Bioroebe project.
  # ========================================================================= #
  TAXONOMY_NCBI_DATABASE_LAST_UPDATE_LOG_FILE =
    "#{::Bioroebe.log_dir?}taxonomy_ncbi_database_last_update.log"

  # ========================================================================= #
  # === NAME_OF_BIO_SHELL
  #
  # This constant can be used as the default prompt for the bioshell
  # component.
  # ========================================================================= #
  NAME_OF_BIO_SHELL = 'BIO SHELL> '

  # ========================================================================= #
  # === DEFAULT_DNA_SEQUENCE
  #
  # This is a default "test" DNA sequence, in the sense that it can be
  # used to quickly test functionality within the bioroebe project.
  #
  # It was added in May 2020, but it may be that we have to remove
  # it at a later time, or move it into a separate .yml file. For
  # the time being, though, it will reside here.
  # ========================================================================= #
  DEFAULT_DNA_SEQUENCE =
    'CGGCCCGATTTGGGTTTCGGAGCGATCGAAATACCAGCACTACCATGAATTCTAT'\
    'ATGGCTGCCGTTCACAGCCTTAATTTTAGGCTTTCCACCTGATCACTCTTTAATC'\
    'TCCATTGTTTCTGGTACGCAGAAATTGACGCTTCCCATTCATTCACGGCTAAAAT'\
    'CAAGGATTCCACCAGAATCGCGGGCCGCGTGGGTGCGCCGTCGACCTCCTCGGCC'\
    'AAATAAGAACGGGCAGGTAAGAGACTAGGGTACTCAAGAT'

  # ========================================================================= #
  # === DEFAULT_LENGTH_FOR_DNA
  #
  # How long our DNA-generated strings should be by default.
  #
  # This may be used by some scripts, so it provides a default value
  # for use in these scripts.
  #
  # 150 nucleotides are the current default.
  # ========================================================================= #
  DEFAULT_LENGTH_FOR_DNA = 150

  require 'bioroebe/constants/tabulator.rb'
  # ========================================================================= #
  # === FIELD_TERMINATOR
  # ========================================================================= #
  FIELD_TERMINATOR = "#{TABULATOR}|#{TABULATOR}"
    MAIN_DELIMITER = DELIMITER = FIELD_TERMINATOR # An alias to the above.

  # ========================================================================= #
  # === BIOROEBE_AT_HOME
  # ========================================================================= #
  BIOROEBE_AT_HOME = "#{RUBY_SRC}bioroebe/lib/bioroebe/"

  # ========================================================================= #
  # === LOCALHOST
  # ========================================================================= #
  LOCALHOST = 'http://localhost/'

  # ========================================================================= #
  # === PATH_TO_THE_RELION_BINARY
  #
  # This constant can be set to determine where relion resides. It is
  # mostly an ad-hoc constant.
  # ========================================================================= #
  PATH_TO_THE_RELION_BINARY =
    '/opt/RELION/relion-1.3/bin/relion'

  # ========================================================================= #
  # === ARRAY_REGISTERED_ACTIONS
  #
  # ARRAY_REGISTERED_ACTIONS becomes @registered_actions.
  # ========================================================================= #
  ARRAY_REGISTERED_ACTIONS = %w(
    to_rna
    to_dna
    rest
    pubmed
    blosum
    restriction
    translate
    quit
    shorten_aminoacid
  )

  # ========================================================================= #
  # === FILE_BIO_LANG
  # ========================================================================= #
  FILE_BIO_LANG =
    "#{USERS_X}data/personal/yaml/bio_lang/bio_lang.md"

  # ========================================================================= #
  # === AC_ELEMENT
  # ========================================================================= #
  # AC_ELEMENT = 4_565 # n nucleotides found in the AC zea mais element.

  # ========================================================================= #
  # === BIOSHELL_FILE
  # ========================================================================= #
  BIOSHELL_FILE = ConvertGlobalEnv[
    '$BIORUBY/shell/shell.rb'
  ] if Object.const_defined?(:ConvertGlobalEnv)

end

if __FILE__ == $PROGRAM_NAME
 pp Bioroebe::DEFAULT_DNA_SEQUENCE
 puts "The log file for the taxonomy-component is at: "\
      "#{Bioroebe::TAXONOMY_NCBI_DATABASE_LAST_UPDATE_LOG_FILE}"  
end