#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file may contain some handy remote URLs, as constants; we could
# also use local "URLs", although this is probably no longer necessary.
# =========================================================================== #
# require 'bioroebe/constants/urls.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === FTP_NCBI_TAXONOMY_DATABASE
  #
  # This constant refers to the taxonomy-database from NCBI. This is the
  # file that can be downloaded from the NCBI homepage (actually, the
  # ftp-listing).
  #
  # Take note that this database, in .tar.gz format, is about 50 MB in 
  # size or even larger these days. So only download it if you really 
  # need it locally.
  # ========================================================================= #
  FTP_NCBI_TAXONOMY_DATABASE =
    'ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz'

  # ========================================================================= #
  # === URL_TO_TAXONOMY_ARCHIVE
  #
  # An "alias" to the above ^^^ constant.
  # ========================================================================= #
  URL_TO_TAXONOMY_ARCHIVE = FTP_NCBI_TAXONOMY_DATABASE

  # ========================================================================= #
  # === NCBI_NUCCORE
  # ========================================================================= #
  NCBI_NUCCORE =
    'https://www.ncbi.nlm.nih.gov/nuccore/'

  # ========================================================================= #
  # === NCBI_GENE
  # ========================================================================= #
  NCBI_GENE =
    'https://www.ncbi.nlm.nih.gov/gene/'

end

if __FILE__ == $PROGRAM_NAME
  _ = "wget #{Bioroebe::FTP_NCBI_TAXONOMY_DATABASE}"
  puts _; system _
end