#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Bioroebe::GUI::Gtk::Alignment
#
# This widget can be used to find the consensus sequence.
# =========================================================================== #
# require 'bioroebe/gui/gtk3/alignment/alignment.rb'
# Bioroebe::GUI::Gtk::Alignment.run
# =========================================================================== #
require 'gtk_paradise/require_gtk3'

module Bioroebe

module GUI

module Gtk

class Alignment < ::Gtk::Box # === Bioroebe::GUI::Gtk::Alignment 

  require 'bioroebe/sequence/alignment.rb'

  require 'gtk_paradise/requires/require_the_base_module.rb'
  include ::Gtk::BaseModule

  # ========================================================================= #
  # === NAMESPACE
  # ========================================================================= #
  NAMESPACE = inspect

  # ========================================================================= #
  # === TITLE
  # ========================================================================= #
  TITLE = 'Alignment'

  # ========================================================================= #
  # === WIDTH
  # ========================================================================= #
  WIDTH = '60%'

  # ========================================================================= #
  # === HEIGHT
  # ========================================================================= #
  HEIGHT = '50%'

  # ========================================================================= #
  # === USE_THIS_FONT
  # ========================================================================= #
  USE_THIS_FONT = :dejavu_condensed_22

  # ========================================================================= #
  # === FONT_FOR_THE_ENTRIES
  #
  # The entries must use a mono-spaced font.
  # ========================================================================= #
  FONT_FOR_THE_ENTRIES = :hack_20

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      commandline_arguments = ARGV,
      run_already           = true
    )
    super(:vertical)
    reset
    set_commandline_arguments(
      commandline_arguments
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    reset_the_internal_variables
    # ======================================================================= #
    # === @configuration
    # ======================================================================= #
    @configuration = [true, __dir__, NAMESPACE]
    # ======================================================================= #
    # === @title
    # ======================================================================= #
    set_title(TITLE)
    # ======================================================================= #
    # === @width
    # ======================================================================= #
    set_width(WIDTH)
    # ======================================================================= #
    # === @height
    # ======================================================================= #
    set_height(HEIGHT)
    set_use_this_font(USE_THIS_FONT)
    use_gtk_paradise_project_css_file
    append_project_CSS_file 
    infer_the_size_automatically
    # ======================================================================= #
    # === @alignment
    # ======================================================================= #
    @alignment = Bioroebe::Alignment.new(nil, :do_not_run_yet)
  end

  # ========================================================================= #
  # === padding?
  # ========================================================================= #
  def padding?
    12
  end

  # ========================================================================= #
  # === border_size?
  # ========================================================================= #
  def border_size?
    6
  end

  # ========================================================================= #
  # === create_the_scrolled_window
  # ========================================================================= #
  def create_the_scrolled_window
    # ======================================================================= #
    # === @scrolled_window
    # ======================================================================= #
    @scrolled_window = gtk_scrolled_window(@text_view)
    @scrolled_window.width_height(300, 400)
    @scrolled_window.css_class('pad4px')
    @scrolled_window.bblack1
  end

  # ========================================================================= #
  # === create_the_text_buffer_and_the_text_view
  # ========================================================================= #
  def create_the_text_buffer_and_the_text_view
    # ======================================================================= #
    # === @text_buffer
    # ======================================================================= #
    @text_buffer = gtk_text_buffer
    # ======================================================================= #
    # === @text_view
    # ======================================================================= #
    @text_view = gtk_text_view(@text_buffer)
  end

  # ========================================================================= #
  # === text_view_widget?
  # ========================================================================= #
  def text_view_widget?
    @text_view
  end

  # ========================================================================= #
  # === do_analyse
  # ========================================================================= #
  def do_analyse(
      _ = @text_buffer.text?
    )
    if _ and !_.empty?
      @alignment.use_this_as_input(_)
      consensus_sequence = @alignment.consensus_sequence?
      @entry_consensus_sequence.set_text(consensus_sequence)
    else
      pop_up_widget(
        over_this_widget: text_view_widget?,
        text:             'Please assign a sequence first.'
      )
    end
  end

  # ========================================================================= #
  # === create_the_entries                           (entry tag, entries tag)
  # ========================================================================= #
  def create_the_entries
    # ======================================================================= #
    # === @entry_consensus_sequence
    # ======================================================================= #
    @entry_consensus_sequence = entry
    @entry_consensus_sequence.bblack1
    @entry_consensus_sequence.center
    @entry_consensus_sequence.use_this_font = FONT_FOR_THE_ENTRIES
    @entry_consensus_sequence.yellow_background
    @entry_consensus_sequence.hint =
      'This entry will contain the consensus sequence, '\
      'derived from the above sequences.'
    # ======================================================================= #
    # === @entry_numbers
    # ======================================================================= #
    @entry_numbers = gtk_entry
    @entry_numbers.use_this_font = FONT_FOR_THE_ENTRIES
    @entry_numbers.hint = 'This area may indicate the numbers for '\
                          'the alignment shown right above.'
  end

  # ========================================================================= #
  # === create_the_buttons                                      (buttons tag)
  # ========================================================================= #
  def create_the_buttons
    # ======================================================================= #
    # === @button_do_analyse
    # ======================================================================= #
    @button_do_analyse = bold_button('_Do analyse')
    @button_do_analyse.bblack1
    @button_do_analyse.on_hover(:lightblue)
    @button_do_analyse.hint = 'Click on this button to '\
      'analyse the given sequence. Make sure to use '\
      'newlines in order to split the sequence at hand.'
    @button_do_analyse.on_clicked {
      do_analyse
      realign_the_numbers_showing_how_long_the_consensus_sequence_is
    }
  end

  # ========================================================================= #
  # === realign_the_numbers_showing_how_long_the_consensus_sequence_is
  # ========================================================================= #
  def realign_the_numbers_showing_how_long_the_consensus_sequence_is
    @entry_numbers.center
    n_characters = @entry_consensus_sequence.text?.size
    @entry_numbers.set_text(return_fill_up_to(n_characters))
  end

  # ========================================================================= #
  # === return_fill_up_to
  # ========================================================================= #
  def return_fill_up_to(i)
    _ = ''.dup
    i.times {|counter|
      counter = (counter+1) % 10
      _ << counter.to_s
    }
    return _
  end

  # ========================================================================= #
  # === connect_skeleton                                        (connect tag)
  # ========================================================================= #
  def connect_skeleton
    abort_on_exception
    top_label = gtk_left_aligned_label(
      'Input the sequences that you wish to compare to one another'
    )
    top_label.center
    minimal(top_label,                 5)
    minimal(@scrolled_window,          2)
    minimal(@button_do_analyse,        2)
    minimal(@entry_consensus_sequence, 2)
    minimal(@entry_numbers,            2)
  end

  # ========================================================================= #
  # === create_skeleton                                          (create tag)
  # ========================================================================= #
  def create_skeleton
    create_the_entries
    create_the_buttons
    create_the_text_buffer_and_the_text_view
    create_the_scrolled_window
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    create_skeleton_then_connect_skeleton
  end

  # ========================================================================= #
  # === Bioroebe::GUI::Gtk::Alignment.run
  # ========================================================================= #
  def self.run(
      i = ARGV
    )
    require 'gtk_paradise/run'
    _ = ::Bioroebe::GUI::Gtk::Alignment.new(i)
    r = ::Gtk.run
    r << _
    r.automatic_size_then_automatic_title
    r.top_left_then_run
  end

end; end; end; end

if __FILE__ == $PROGRAM_NAME
  Bioroebe::GUI::Gtk::Alignment.run
end