#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Bioroebe::PalindromeFinder
#
# This class shall find a palindrome in a sequence.
#
# It currently appears to be broken. I will keep it for the time,
# but it should really be fixed (or removed) eventually (April 2020). 
# =========================================================================== #
# require 'bioroebe/palindromes/palindrome_finder.rb'
# =========================================================================== #
require 'bioroebe/base/commandline_application/commandline_application.rb'

module Bioroebe

class PalindromeFinder < ::Bioroebe::CommandlineApplication

  # ========================================================================= #
  # === DEFAULT_SEARCH_STRING
  # ========================================================================= #
  DEFAULT_SEARCH_STRING = 'RENTNER reliefpfeiler anna ATCTA'

  # ========================================================================= #
  # === initialize
  #
  # Can also invoke it via rpalindrome.
  # Currently we can find only palindromes which have odd numbers
  # ========================================================================= #
  def initialize(
      string_to_search   = 'RENTNER reliefpfeiler anna ATCTA',
      do_display_results = false
    )
    case do_display_results
    when :display_results
      do_display_results = true
    end
    set_string_to_search(
      string_to_search
    )
    reset
    run if do_display_results
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    super()
    # ======================================================================= #
    # === @array_found_palindrome
    #
    # This array will keep the palindromes.
    # ======================================================================= #
    @array_found_palindrome = []
    # ======================================================================= #
    # === @last_char
    # ======================================================================= #
    @last_char    = ''
    # ======================================================================= #
    # === @current_char
    # ======================================================================= #
    @current_char = ''
    # ======================================================================= #
    # === @next_char
    # ======================================================================= #
    @next_char    = ''
    # ======================================================================= #
    # === @longest_palindrome
    # ======================================================================= #
    @longest_palindrome = nil # if we did found a palindrome, this counter
                              # tells us how many chars it has.
  end

  # ========================================================================= #
  # === set_string_to_search
  # ========================================================================= #
  def set_string_to_search(
      i = DEFAULT_SEARCH_STRING
    )
    i = i.first if i.is_a? Array
    i = DEFAULT_SEARCH_STRING if i.nil?
    i = DEFAULT_SEARCH_STRING if i.is_a?(Array) and i.empty?
    @string_to_search = i
  end

  # ========================================================================= #
  # === find_seeds
  #
  # This method finds every seed to a palindrome in a given string.
  #
  # It will return an array with their respective positions.
  # ========================================================================= #
  def find_seeds
    index = 0
    array = []
    @string_to_search.each_char { |char|
      current_char = char
      next_char = @string_to_search[index+1, 1]
      last_char = @string_to_search[index-1, 1]
      if next_char == ' ' or last_char == ' ' or current_char == ' ' # Skip empty lines
        index += 1
        next
      else
        array << index if next_char == last_char
        index += 1
      end
    }
    return array
  end

  # ========================================================================= #
  # === find_complex_palindromes
  #
  # Please invoke this method only after find_simple_palindromes has 
  # been invoked. This is not very smart right now, but i dont wanna 
  # change it yet. :P
  #
  # May be half deprecated hmmm.
  # ========================================================================= #
  def find_complex_palindromes
    @array_found_palindrome.each { |palindrome|
      palindrome_start_pos = palindrome[1]
      palindrome_content   = palindrome[0] 
      if @string_to_search[palindrome_start_pos-1,1] == @string_to_search[palindrome_start_pos+palindrome_content.size+1,1]
        e sfancy(
            @string_to_search[palindrome_start_pos - 1, 1]+
            palindrome_content+
            @string_to_search[palindrome_start_pos+palindrome_content.size+1,1]
          )
      end 
    }
  end

  # ========================================================================= #
  # === find_palindrome
  #
  # What is the property of a palindrome?
  #
  #   anna, otto, abcba, eeegeee, RENTNER, Reliefpfeiler
  #
  # Basically a property.
  #
  # The "middle" part is normally static.
  # ========================================================================= #
  def find_simple_palindromes
    index = 1
    string_size = @string_to_search.size
    loop {
      if @string_to_search[index, 1] == ' '
        # safe guard to intercept (and ignore) empty positions (i.e. space char)
      elsif @string_to_search[index-1, 1] == ' ' and
            @string_to_search[index+1,1]  == ' '
        # Here too.
      else
        @last_char    = @string_to_search[index-1,1]
        @current_char = @string_to_search[index,1]
        @next_char    = @string_to_search[index+1,1]
        if @next_char == @last_char
          palindrome = @last_char+@current_char+@next_char
          pp palindrome
          # The format for @array_found_palindrome is:
          #   0 sequence
          #   1 start_position
          #   2 end_position
          @array_found_palindrome << [ palindrome, index-1 ]
        end
      end
      index += 1
      break if string_size == index
    }
    e
  end
  
  # ========================================================================= #
  # === display_results
  # ========================================================================= #
  def display_results
    e "#{rev}Looking for results in the string "\
      "`#{sfancy(@string_to_search)}#{rev}`:"
    if @array_found_palindrome.empty?
      erev 'No Palindrome was found in this string.'
    else
      @array_found_palindrome.each_with_index { |palindrome, index|
        erev 'Palindrome '+steelblue(index.to_s)
        erev "    #{palindrome.first}"
      }
    end
  end

  # ========================================================================= #
  # === run
  # ========================================================================= #
  def run
    display_results
  end

end; end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    x = Bioroebe::PalindromeFinder.new(ARGV, :display_results)
    pp x.find_seeds
  else
    Bioroebe::PalindromeFinder.new(ARGV, :display_results)
  end
end # palindromefinder otto