#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/regexes/regexes.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/codons/start_codons.rb'

  # ========================================================================= #
  # === Bioroebe.regex_with_startcodons
  #
  # This toplevel-method will return the regex for a start codon.
  #
  # Usually this means that we will return /ATG|AUG/ so we also
  # match towards the mRNA variant. It was created precisely
  # because the user may often wish to identify any start codon,
  # no matter if it is kept in RNA or in DNA.
  # ========================================================================= #
  def self.regex_with_startcodons(
      use_this_start_codon = ::Bioroebe.start_codon?
    )
    start_codon_as_mRNA = use_this_start_codon.
                          tr('T','U') 
    regex = /(#{use_this_start_codon}|#{start_codon_as_mRNA})/i
    return regex # Return the regex here.
  end

end

if __FILE__ == $PROGRAM_NAME
  p Bioroebe.regex_with_startcodons
end # regexwithstartcodons