#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === require_yaml.rb
#
# This file can be used to either load up and subsequently use Psych,
# or load up and then use the older Syck gem.
#
# Thus, this file allows us some flexibility for the bioroebe-project.
#
# Since as of the year 2019, using psych should have no real major
# drawbacks really, so the BioRoebe project now defaults to psych
# past 2019.
# =========================================================================== #
# require 'bioroebe/requires/require_yaml.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === @use_this_yaml_engine
  #
  # Please keep the value to this variable as a Symbol at all times.
  #
  # The two possible variants are :syck and :psych, currently.
  #
  # :psych is the default since some time.
  # ========================================================================= #
  @use_this_yaml_engine = :psych # Our default yaml engine to use.

  # ========================================================================= #
  # === Bioroebe.use_which_yaml_engine?
  #
  # Query-method to find out which yaml-engine is used for the
  # BioRoebe project.
  # ========================================================================= #
  def self.use_which_yaml_engine?
    @use_this_yaml_engine
  end

  # ========================================================================= #
  # === Bioroebe.use_which_yaml_engine?
  #
  # Query method to determine which yaml engine is in use for the Bioroebe
  # project.
  # ========================================================================= #
  def self.set_use_this_yaml_engine=(i = :default)
    case i # case tag
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default
      i = :psych
    end
    @use_this_yaml_engine = i
  end; self.instance_eval { alias set_yaml_engine set_use_this_yaml_engine= } # === Bioroebe.set_yaml_engine

  # ========================================================================= #
  # === Bioroebe.load_psych_or_syck
  # ========================================================================= #
  def self.load_psych_or_syck(
      i = @use_this_yaml_engine
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         :psych,
         nil
      require 'yaml' # This should suffice as-is.
    else # Else give more information to the user how to get syck.
      begin
        require 'syck'
      rescue LoadError
        # =================================================================== #
        # Inform the user how syck can be installed.
        # =================================================================== #
        e
        e 'syck is not available. It can be installed via:'
        e
        e '  gem install syck'
        e
      end
    end
  end

  # ========================================================================= #
  # Load psych or syck at once, whenever this file is loaded. By default
  # this will load up psych past 2019.
  # ========================================================================= #
  load_psych_or_syck

end