#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/ad_hoc_task.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/e.rb'
  require 'bioroebe/toplevel_methods/file_and_directory_related_actions.rb'

  # ========================================================================= #
  # === Bioroebe.ad_hoc_task
  #
  # This method can be used to specifically run an "ad-hoc" task.
  #
  # An ad-hoc task is something that we just quickly "hack" together,
  # in order to solve some existing bioinformatics-related problem.
  #
  # Presently, in May 2021, this was for a university course that
  # required us to work with MEGA X and compare different proteins
  # from a phylogenetics point of view.
  # ========================================================================= #
  def self.ad_hoc_task(
      this_file = '/root/Bioroebe/table_ids.md'
    )
    require 'bioroebe/fasta_and_fastq/download_fasta.rb'
    require 'bioroebe/fasta_and_fastq/simplify_fasta_header/simplify_fasta_header.rb'
    if this_file.is_a? Array
      this_file = this_file.join(' ')
    end
    cd ::Bioroebe.log_dir? # Make sure we are in the log-directory.
    e 'Now downloading some FASTA files, based on this file: '+
      this_file
    # ======================================================================= #
    # (1) Download the remote FASTA dataset
    # ======================================================================= #
    download_fasta this_file
    # ======================================================================= #
    # (2) cd into the fasta directory
    # ======================================================================= #
    cd ::Bioroebe.log_dir?+'fasta/'
    # ======================================================================= #
    # (3) batch rename all .fasta files next via simplify-fasta-header.
    # ======================================================================= #
    all_files = Dir['*.fasta']
    all_files.each {|this_fasta_file|
      Bioroebe.overwrite_fasta_header(this_fasta_file)
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.ad_hoc_task
end # adhoctask