#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/calculate_n50_value.rb'
# Bioroebe.calculate_n50_value(ARGV)
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.calculate_n50_value
  #
  # This method will calculate the N50 value of the given input. The input
  # to this method should be a sorted Array.
  # ========================================================================= #
  def self.calculate_n50_value(
      i = [
        1989, 1934, 1841,
        1785, 1737, 1649,
        1361,  926,  848,
         723
      ]
    )
    # ======================================================================= #
    # The following conversion is necessary because ARGV will contain only
    # String objects, not integer-values.
    # ======================================================================= #
    i.map! {|entry| entry.to_i }
    calculate_sum_for_the_loop = 0
    sum = i.sum
    half = sum / 2.0
    find_the_proper_contig = nil
    i.each {|this_number|
      calculate_sum_for_the_loop += this_number
      # ===================================================================== #
      # Compare the temporary sum with the half-sum.
      # ===================================================================== #
      if calculate_sum_for_the_loop > half
        find_the_proper_contig = this_number
        break
      end
    }
    return find_the_proper_contig
  end

end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    pp Bioroebe.calculate_n50_value
    array_to_test = [1252, 919, 917, 765, 711, 690, 632, 516, 463, 218]
    pp array_to_test
    pp Bioroebe.calculate_n50_value(array_to_test)
  else
    pp Bioroebe.calculate_n50_value(ARGV)
  end
end # calculaten50value