#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/cat.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/convert_global_env.rb'
  require 'bioroebe/toplevel_methods/e.rb'
  require 'bioroebe/colours/colours.rb'
  require 'bioroebe/colours/rev.rb'
  require 'bioroebe/colours/sfile.rb'
  require 'bioroebe/colours/sdir.rb'

  # ========================================================================= #
  # === Bioroebe.cat                                                (cat tag)
  #
  # A variant of cat to use here.
  # ========================================================================= #
  def self.cat(i = nil)
    if i
      i = convert_global_env(i) if i.include? '$'
      i = Dir['*'][i.to_i - 1] if i =~ /^\d+$/
    end
    if i.nil?
      erev 'Please provide an argument to Bioroebe.cat() (the name of a file)'
    # ======================================================================= #
    # === Handle directories next
    # ======================================================================= #
    elsif File.directory? i
      erev "We can not read from `#{sdir(i)}#{rev}` as it is a directory."
    # ======================================================================= #
    # Else the File will exist in this clause:
    # ======================================================================= #
    elsif File.exist?(i)
      _ = File.extname(i).delete('.')
      case _ # case tag
      # ===================================================================== #
      # === fasta
      # ===================================================================== #
      when 'fasta',
           'fa'
        e 'This is a fasta file, so rather than cat-ing the content,'
        e 'we will send this dataset to the ParseFasta class.'
        require 'bioroebe/fasta_and_fastq/parse_fasta/parse_fasta.rb'
        Bioroebe::ParseFasta.new(i)
      else # The default here.
        e "Now displaying the file `#{sfile(i)}`."
        # e File.read(i)
        # ^^^ Or we could use the above. We have to reconsider this one day.
        File.readlines(i).each {|line| e "  #{line.chomp}" } # With a bit of padding.
      end
    else # else the file does not exist.
      e "#{swarn('Trying to display the file `')}#{sfile(i)}#{swarn('`')}"
      e swarn('but it does not exist.')
    end
  end

  # ========================================================================= #
  # === cat
  # ========================================================================= #
  def cat(i)
    ::Bioroebe.cat(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.cat '/etc/wgetrc' # This is just for testing purposes really.
end # bioroebe_cat