#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/complement.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.complement
  #
  # This method will return the complementary DNA strand.
  #
  # We will use possibilities though.
  #
  # Usage example:
  #
  #   Bioroebe.complement 'ATGGGTCCC' # => "TACCCAGGG"
  #
  # ========================================================================= #
  def self.complement(
      i = nil
    )
    require 'bioroebe/constants/nucleotides.rb'
    # ======================================================================= #
    # Refer to the main Hash.
    # ======================================================================= #
    hash = HASH_DNA_NUCLEOTIDES
    result = ''.dup
    i = i.first if i.is_a? Array
    if i
      if File.exist?(i)
        i = File.readlines(i).join(' ').strip
      end
      i.each_char { |char|
        char = char.upcase
        if hash.has_key? char
          result << hash[char]
        else
          case char.downcase # case tag
          when 'n' # Means any.
            result << '(A/T/G/C)'
          when 'r' # Means a purine.     (larger)
            result << '(A/G)'
          when 'y' # Means a pyrimidine. (smaller)
            result << '(T/C)'
          end
        end
      }
      return result
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.complement(ARGV) # => "TACCCAGGG"
end # bioroebecomplement TACCCAGGG