#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/databases.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/constants/database_constants.rb'

  # ========================================================================= #
  # === Bioroebe.set_pgpassword                                (password tag)
  #
  # Set the password for Postgresql here. As input to this method you
  # can pass the password for the postgreSQL database.
  # ========================================================================= #
  def self.set_pgpassword(
      new_password = POSTGRE_PASSWORD
    )
    ENV['PGPASSWORD'] = new_password
  end; self.instance_eval { alias set_password set_pgpassword } # === Bioroebe.set_password

  # ========================================================================= #
  # === Bioroebe.run_sql_query
  #
  # This method will run a SQL query using --command.
  #
  # It will return the result as well.
  #
  # The second argument determines whether we will be verbose or not.
  #
  # The result, however, will always be displayed.
  # ========================================================================= #
  def self.run_sql_query(
      i,
      be_verbose           = true,
      optional_append_this = ''
    )
    set_pgpassword
    i = i.to_s
    _ = ''
    if is_on_roebe?
      _ << POSTGRE_LOGIN_COMMAND_HOME
    else
      _ << POSTGRE_LOGIN_COMMAND
    end
    case be_verbose
    when :silent
      be_verbose = false
    end
    case optional_append_this
    # ======================================================================= #
    # === :bare
    # ======================================================================= #
    when :bare,
         :tuples
      optional_append_this = ' --tuples-only'
    end
    _ << optional_append_this.to_s
    _ << ' --command="'
    _ << i
    _ << '"' unless _.end_with? '"'
    result = `#{_}`.chomp
    erev _      if be_verbose
    erev result if be_verbose
    return result
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.run_sql_query 'foo bar'  
end