#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/digest.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.digest_this_dna
  #
  # Usage examples:
  #
  #   Bioroebe.digest_this_dna(:lambda_genome, with: :EcoRI)
  #   Bioroebe.digest_this_dna("/root/Bioroebe/fasta/NC_001416.1_Enterobacteria_phage_lambda_complete_genome.fasta", with: :EcoRI)
  #
  # ========================================================================= #
  def self.digest_this_dna(
      this_DNA_sequence, hash = {}
    )
    require 'bioroebe/toplevel_methods/e.rb'
    require 'bioroebe/constants/files_and_directories.rb'
    require 'bioroebe/fasta_and_fastq/parse_fasta/parse_fasta.rb'
    restriction_enzymes = Bioroebe.load_and_return_the_restriction_enzymes
    this_restriction_enzyme = nil
    nucleotide_sequence = nil
    if this_DNA_sequence.is_a? Array
      this_DNA_sequence = this_DNA_sequence.first
    end
    if this_DNA_sequence.is_a?(String) and File.exist?(this_DNA_sequence)
      nucleotide_sequence = ::Bioroebe::ParseFasta.new(this_DNA_sequence).sequence?
    end
    # ======================================================================= #
    # === Handle the hash next (and ensure that it is a Hash)
    # ======================================================================= #
    if hash.is_a? Hash
      # ===================================================================== #
      # === :with
      # ===================================================================== #
      if hash.has_key? :with
        this_restriction_enzyme = hash.delete(:with).to_s
      end
    end
    target_sequence = restriction_enzymes[this_restriction_enzyme].dup
    if target_sequence =~ /\d$/ # If it ends with a number.
      target_sequence.chop!
      target_sequence.strip!
    end
    if nucleotide_sequence.include?(target_sequence)
      e 'Yes, this sequence is there.'
      scanned = nucleotide_sequence.scan(
        /#{target_sequence}/
      )
      e "It can be found #{scanned.size.to_s} times."
      sub_sequences = nucleotide_sequence.split(/#{target_sequence}/)
      sub_sequences.sort_by {|entry| entry.size }.reverse.each {|sequence|
        pp sequence.size
      }
    else
      e 'Nothing found.'
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.digest_this_dna(
    "/home/Temp/bioroebe/fasta/NC_001416.1_Enterobacteria_phage_lambda_complete_genome.fasta",
    with: :EcoRI
  )
end # rb digest.rb