#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/exponential_growth.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.calculate_exponential_growth
  #
  # This method can be used to calculate how many bacteria will exist after
  # n cell divisions (provided that we know, and supply to this method,
  # how many bacteria existed when we started our calculation).
  # ========================================================================= #
  def self.calculate_exponential_growth(
      number_of_cells     = 10,
      number_of_divisions = 10
    )
    if number_of_cells.nil?
      number_of_cells = 10 # Default value.
    end
    if number_of_divisions.nil?
      number_of_divisions = 10 # Default value.
    end
    # ======================================================================= #
    # === Hashes
    #
    # Handle Hash as input given.
    # ======================================================================= #
    if number_of_cells.is_a? Hash
      if number_of_cells.has_key? :n_divisions
        number_of_divisions = number_of_cells.delete(:n_divisions)
      end
      if number_of_cells.has_key? :number_of_cells
        number_of_cells = number_of_cells.delete(:number_of_cells)
      elsif number_of_cells.has_key? :n_cells
        number_of_cells = number_of_cells.delete(:n_cells)
      end
    end
    # ======================================================================= #
    # We need numbers, aka integers - there are no "1.3" cells.
    # ======================================================================= #
    number_of_cells     = number_of_cells.to_i
    number_of_divisions = number_of_divisions.to_i
    total_amount_of_cells = 0
    total_amount_of_cells = number_of_cells * (2 ** number_of_divisions)
    return total_amount_of_cells
  end

  # ========================================================================= #
  # === Bioroebe.calculate_original_amount_of_cells_of_exponential_growth
  #
  # The first argument, number_of_cells, means "how many cells do we have
  # now/currently". This is necessary, in order to calculate how many
  # cells we used to have initially.
  # ========================================================================= #
  def self.calculate_original_amount_of_cells_of_exponential_growth(
      number_of_cells     = 1600, # 1600 cells to start with.
      number_of_divisions =    5  #    5 generations by default.
    )
    number_of_cells     = number_of_cells.to_i
    number_of_divisions = number_of_divisions.to_i
    initial_amount_of_cells_was = 0
    initial_amount_of_cells_was = number_of_cells / ( 2 ** number_of_divisions )
    return initial_amount_of_cells_was
  end

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.calculate_exponential_growth(ARGV[0], ARGV[1])
  # puts Bioroebe.calculate_original_amount_of_cells_of_exponential_growth()
end # calculateexponentialgrowth 10 10