#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/extract.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/file_and_directory_related_actions.rb'
  require 'bioroebe/toplevel_methods/esystem.rb'
  require 'bioroebe/toplevel_methods/verbose.rb'

  # ========================================================================= #
  # === Bioroebe.extract
  #
  # This method can be used to quickly extract a local archive.
  # ========================================================================= #
  def self.extract(
      i = ARGV
    )
    require 'bioroebe/colours/sfancy.rb'
    require 'bioroebe/colours/swarn.rb'
    if i.is_a? Array
      i = i.join(' ').strip
    end
    unless i.include?('/')
      unless File.exist? i
        i = return_pwd+File.basename(i)
      end
    end
    if File.exist? i
      case i
      when /bz2$/
        _ = "tar -xfv #{i}"
      when /xz$/
        _ = "tar -xvf #{i}"
      end
      if be_verbose?
        e "Now extracting `#{sfancy((i).squeeze('/'))}`."
        esystem(_)
        e 'Done extracting!'
      else
        system _
      end
    else
      ewarn "Can not extract #{sfile(i)} because it does "\
            "not appear to exist."
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.extract(ARGV)
end # ruby extract.rb /home/x/src/htop/htop-3.0.5.tar.xz