#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/frequencies.rb'
# Bioroebe.aminoacid_frequency
# =========================================================================== #
module Bioroebe

  require 'bioroebe/project/project.rb'
  require 'bioroebe/toplevel_methods/e.rb'
  require 'bioroebe/colours/colours.rb'

  # ========================================================================= #
  # === Bioroebe.calculate_the_frequencies_of_this_species
  # ========================================================================= #
  def self.calculate_the_frequencies_of_this_species(
      i = :homo_sapiens
    )
    require 'bioroebe/sequence/dna.rb'
    require 'yaml'
    if i and i.is_a?(Array) and i.empty?
      i << :homo_sapiens
    end
    hash_aminoacids = {}
    hash_aminoacids.default = 0
    if i.is_a? Array
      i = i.first
    end
    case i.to_sym
    # ======================================================================= #
    # === :homo_sapiens
    # ======================================================================= #
    when :homo_sapiens,
         :homo,
         :human
      i = "#{::Bioroebe.project_base_directory?}"\
          "codon_tables/frequencies/9606_Homo_sapiens.yml"
    end
    hash = YAML.load_file(i)
    # "GAC"=>25.1
    hash.each_pair {|key, value|
      this_aminoacid = Bioroebe.to_aa(key)
      hash_aminoacids[this_aminoacid] += value
    }
    e
    # ======================================================================= #
    # Convert it into percent:
    # ======================================================================= #
    hash_aminoacids.each_pair {|key, value_for_percentage|
      value_for_percentage = ((value_for_percentage * 100.0) / 1000.0).round(3).to_s
      value_for_percentage = '%.2f' % value_for_percentage
      e '  '+
        steelblue(key).to_s+' '+
        royalblue(
          value_for_percentage.rjust(6)+'%'
        )
    }
    e
  end

  # ========================================================================= #
  # === Bioroebe.aminoacid_frequency
  #
  # Usage example:
  #
  #   Bioroebe.aminoacid_frequency('MVTDEGAIYFTKDAARNWKAAVEETVSATLNRTVSSGITGASYYTGTFST')
  #
  # Would yield the following Hash:
  #
  #   {"M"=>1, "V"=>4, "T"=>9, "D"=>2, "E"=>3, "G"=>4, "A"=>7, "I"=>2, "Y"=>3, "F"=>2, "K"=>2, "R"=>2, "N"=>2, "W"=>1, "S"=>5, "L"=>1}
  #
  # ========================================================================= #
  def self.aminoacid_frequency(
      of_this_sequence = ''
    )
    if of_this_sequence.is_a? Array
      of_this_sequence = of_this_sequence.first 
    end
    chars = of_this_sequence.split(//)
    hash = {}
    hash.default = 0
    chars.each {|this_char| hash[this_char] += 1 }
    return hash
  end; self.instance_eval { alias aminoacid_frequencies aminoacid_frequency } # === Bioroebe.aminoacid_frequencies

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.calculate_the_frequencies_of_this_species(ARGV)
  pp Bioroebe.aminoacid_frequency('MVTDEGAIYFTKDAARNWKAAVEETVSATLNRTVSSGITGASYYTGTFST')
end # calculatethefrequenciesofthisspecies