#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/hamming_distance.rb'
# Bioroebe.hamming_distance('ATCG','ATCC')
# puts Bioroebe.hamming_distance(ARGV[0], ARGV[1])
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.hamming_distance
  #
  # This method will return an Integer, aka a number, which represents
  # the hamming distance between two sequences of equal length. This
  # will state how many differences exist between two same-sized
  # sequences (aka sequences that have the same length).
  #
  # Do note that a second implementation may exist for the hamming
  # distance, in the Bioroebe project.
  #
  # Usage example:
  #
  #   Bioroebe.hamming_distance('ATCG','ATCC') # => 1
  #
  # ========================================================================= #
  def self.hamming_distance(
      sequence1 = 'ATCG',
      sequence2 = 'ATCC'
    )
    if sequence1.nil?
      e 'Please provide a sequence (String) as input to this method.'
      return
    end
    if sequence1.is_a? String
      sequence1 = sequence1.split(//)
    end
    if sequence2.is_a? String
      sequence2 = sequence2.split(//)
    end
    array_sequence1 = [sequence1].flatten
    array_sequence2 = [sequence2].flatten
    # ======================================================================= #
    # Zip the two sequences together, then reduce this Array of
    # zipped values to an integer value, which will be returned.
    # ======================================================================= #
    zipped_array = array_sequence1.zip(array_sequence2)
    hamming_value = 0
    zipped_array.each { |left, right|
      hamming_value += 1 unless left == right
    }
    return hamming_value
  end; self.instance_eval { alias hamming hamming_distance } # === Bioroebe.hamming

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.hamming_distance(ARGV[0], ARGV[1])
end # hammingdistancemodulemethod GAGCCTACTAACGGGAT CATCGTAATGACGGCCT
    # hammingdistancemodulemethod ATC ATT