#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file may contain code that can be used to "infer" something.
#
# The primary use case is to infer whether a given sequence is DNA,
# RNA or an aminoacid sequence.
# =========================================================================== #
# require 'bioroebe/toplevel_methods/infer.rb'
# Bioroebe.infer_type_from_this_sequence
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.infer_type_from_this_sequence
  #
  # This method will try to infer the type from a given sequence.
  #
  # Valid return types are the following symbols:
  #
  #   :dna
  #   :rna
  #   :protein
  #
  # Note that this may not work 100% reliably, so do not depend too
  # much on this method working absolutely perfect.
  # ========================================================================= # 
  def self.infer_type_from_this_sequence(
      i = 'ATGGTACGACAC'
    )
    if i.is_a? Array
      i = i.join
    end
    type = :dna # This is the default - DNA.
    # ======================================================================= #
    # === :rna
    # ======================================================================= #
    if i.include? 'U'
      type = :rna
    # ======================================================================= #
    # === :dna
    # ======================================================================= #
    elsif i =~ /^[ATCG]+$/
      type = :dna 
    # ======================================================================= #
    # === :protein
    # ======================================================================= #
    else # else assume this to be a protein.
      type = :protein
    end
    return type
  end; self.instance_eval { alias infer_the_type_from_this_sequence infer_type_from_this_sequence } # === Bioroebe.infer_the_type_from_this_sequence

end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    # In this case test this.
    pp Bioroebe.infer_type_from_this_sequence('ATCGATCATTCGATGCAGTCA')   # => :dna
    pp Bioroebe.infer_type_from_this_sequence('UCAGUCAGUCAGUCAGUACGUCA') # => :rna
    pp Bioroebe.infer_type_from_this_sequence('ATGGGVVVIIILLP')          # => :protein
  else
    pp Bioroebe.infer_type_from_this_sequence(ARGV)
  end
end # rb infer.rb ATCAGCTAGCATCA