#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/leading_five_prime_and_trailing_three_prime.rb'
# Bioroebe.leading_five_prime(ARGV)
# Bioroebe.trailing_three_prime(ARGV)
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.leading_five_prime
  #
  # This method will output the leading 5' part of a nucleotide sequence,
  # like a header. In this context, the most commonly usage case for
  # this method is to display a leading "5'-" part to a sequence such
  # as "ATCGATCG".
  #
  # The hyphen refers to the '-' character which may be used by this
  # method. This is optional, though - have a look at the third
  # argument to this method called `use_hyphen?.
  # ========================================================================= #
  def self.leading_five_prime(
      i                 = '', # The default to '' is important.
      get_rid_of_spaces = false,
      use_hyphen        = true # By default this method will use '-' characters.
    )
    case get_rid_of_spaces
    # ======================================================================= #
    # === :no_spaces
    # ======================================================================= #
    when :no_spaces,
         :get_rid_of_spaces
      get_rid_of_spaces = true
    end
    if i.is_a? Array
      i = i.join.strip
    end
    if use_hyphen
      _ = "5' - #{i}" # ← This here is the header-tag.
    else # This variant is a bit simpler and shorter.
      _ = "5' #{i}" # ← This here is the header-tag.
    end
    if get_rid_of_spaces
      _ = _.dup if _.frozen?
      _.delete!(' ')
    end
    return _
  end; self.instance_eval { alias five_prime               leading_five_prime } # === Bioroebe.five_prime
       self.instance_eval { alias leading_5_prime          leading_five_prime } # === Bioroebe.leading_five_prime
       self.instance_eval { alias leader                   leading_five_prime } # === Bioroebe.leader
       self.instance_eval { alias lead_five_prime          leading_five_prime } # === Bioroebe.lead_five_prime
       self.instance_eval { alias return_five_prime_header leading_five_prime } # === Bioroebe.return_five_prime
       self.instance_eval { alias leading_five             leading_five_prime } # === Bioroebe.return_five

  # ========================================================================= #
  # === Bioroebe.trailing_three_prime
  #
  # This method will output the trailing 3' part, like a header (or rather,
  # a trailer).
  # ========================================================================= #
  def self.trailing_three_prime(
      i                 = '', # The default to '' is important.
      get_rid_of_spaces = false,
      use_hyphen        = true
    )
    case get_rid_of_spaces
    # ======================================================================= #
    # === :no_spaces
    # ======================================================================= #
    when :no_spaces,
         :get_rid_of_spaces,
         :eliminate_spaces
      get_rid_of_spaces = true
    end
    if i.is_a? Array
      i = i.join.strip
    end
    if use_hyphen
      _ = "#{i} - 3'" # ← This here is the header-tag.
    else # This variant is a bit simpler and shorter.
      _ = "#{i} 3'"   # ← This here is the header-tag.
    end
    _.delete!(' ') if get_rid_of_spaces
    return _
  end; self.instance_eval { alias three_prime               trailing_three_prime } # === Bioroebe.three_prime
       self.instance_eval { alias trailing_3_prime          trailing_three_prime } # === Bioroebe.trailing_three_prime
       self.instance_eval { alias leader                    trailing_three_prime } # === Bioroebe.leader
       self.instance_eval { alias lead_three_prime          trailing_three_prime } # === Bioroebe.lead_three_prime
       self.instance_eval { alias return_three_prime_header trailing_three_prime } # === Bioroebe.return_three_prime
       self.instance_eval { alias trailing_three            trailing_three_prime } # === Bioroebe.return_three

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.leading_five_prime(ARGV)
  puts Bioroebe.trailing_three_prime(ARGV)
  puts Bioroebe.leading_five_prime(ARGV, :eliminate_spaces)
  puts Bioroebe.trailing_three_prime(ARGV, :eliminate_spaces)
end # leadingfiveprime   ATGCT
    # trailingthreeprime ATGCT