#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.min3
  # ========================================================================= #
  def self.min3(a, b, c)
    if a < b && a < c
      a
    elsif b < c
      b
    else
      c
    end
  end

  # ========================================================================= #
  # === Bioroebe.levenshtein_distance
  # ========================================================================= #
  def self.levenshtein_distance(str1, str2)
    n = str1.length
    m = str2.length

    return m if n.zero?
    return n if m.zero?

    x = nil
    d = (0 .. m).to_a # Get the distance.

    # ======================================================================= #
    # To avoid duplicating an enumerable object, create it outside of the
    # loop.
    # ======================================================================= #
    str2_codepoints = str2.codepoints

    str1.each_codepoint.with_index(1) { |char1, i|
      j = 0
      while j < m
        cost = (char1 == str2_codepoints[j]) ? 0 : 1
        x = min3(
          d[j + 1] + 1, # insertion
          i + 1,        # deletion
          d[j] + cost   # substitution
        )
        d[j] = i
        i = x
        j += 1
      end
      d[m] = x
    }
    x # Return the cost here.
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e 'Difference between "shevy" and "chevy" is: '+
     Bioroebe.levenshtein_distance('shevy', 'chevy').to_s # => 1  
end # rb levensthein.rb