#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file keeps track of where we will store output generated by
# the Bioroebe project.
# =========================================================================== #
# require 'bioroebe/toplevel_methods/log_directory.rb'
# Bioroebe.download_dir?
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === @log_directory
  #
  # Keep track where to log output into, through this variable.
  # ========================================================================= #
  @log_directory = nil

  # ========================================================================= #
  # === Bioroebe.log_directory?
  #
  # This method will keep track over where we will store output,
  # related to the Bioroebe project.
  # ========================================================================= #
  def self.log_directory?
    @log_directory
  end; self.instance_eval { alias log_dir                log_directory? } # === Bioroebe.log_dir
       self.instance_eval { alias store_here?            log_directory? } # === Bioroebe.store_here?
       self.instance_eval { alias save_dir?              log_directory? } # === Bioroebe.save_dir?
       self.instance_eval { alias log_dir?               log_directory? } # === Bioroebe.log_dir?
       self.instance_eval { alias project_log_directory? log_directory? } # === Bioroebe.project_log_directory?

  require 'bioroebe/constants/base_directory.rb'
  # ========================================================================= #
  # === Bioroebe.set_log_dir
  #
  # This method will set the log-directory for the Bioroebe project.
  #
  # Usually this will be in the user's home directory, and then a
  # 'Bioroebe/' suffix appended. The default input-value is the
  # symbol :default.
  #
  # The user can set to use another log-directory through this method.
  # ========================================================================= #
  def self.set_log_directory(
      i = :default
    )
    case i # case tag
    # ======================================================================= #
    # === :default
    #
    # This is the default value.
    # ======================================================================= #
    when :default
      i = ::Bioroebe.base_directory?
    end
    unless i.end_with? '/'
      i = i.dup if i.frozen?
      i << '/'
    end
    @log_directory = i
  end; self.instance_eval { alias set_log_dir set_log_directory } # === Bioroebe.set_log_dir

  # ========================================================================= #
  # === Initialize it at once, with the default value.
  # ========================================================================= #
  _ = :default # Uses the default path.
  if ENV.has_key? 'BIOROEBE_DEFAULT_LOG_DIRECTORY'
    _ = ENV['BIOROEBE_DEFAULT_LOG_DIRECTORY']
  end
  set_log_directory(_)

  # ========================================================================= #
  # === Bioroebe.download_directory?
  # ========================================================================= #
  def self.download_directory?
    "#{@log_directory}Downloads/"
  end; self.instance_eval { alias download_dir? download_directory? } # === Bioroebe.download_dir?

  # ========================================================================= #
  # === LOCAL_DIRECTORY_FOR_UNIPROT
  #
  # This denotes the directory for uniprot-files.
  # ========================================================================= #
  LOCAL_DIRECTORY_FOR_UNIPROT = "#{@log_directory}uniprot/"

  # ========================================================================= #
  # === Bioroebe::AUTOGENERATED_SQL_FILES_DIR
  # ========================================================================= #
  AUTOGENERATED_SQL_FILES_DIR =
    "#{@log_directory}autogenerated_sql_files/" # Where to store .sql files.

  # ========================================================================= #
  # === Bioroebe.fasta_directory?
  #
  # This method will return a path such as "/root/Bioroebe/fasta/".
  # ========================================================================= #
  def self.fasta_directory?
    "#{::Bioroebe.log_directory?}fasta/"
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e Bioroebe.log_directory?
  e Bioroebe.download_directory?  
end