#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/longest_common_substring.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.longest_common_substring
  #
  # This method will return the longest common substring. There may be
  # more than one solution, though.
  #
  # Note that this method was specifically written for Rosalind, so it
  # may not be too overly useful for most general tasks. For example,
  # it is not optimised for speed, so you should probably not use
  # it for very long sequences.
  # ========================================================================= #
  def self.longest_common_substring(sequences)
    if sequences and !sequences.empty? and File.file?(sequences.first)
      require 'bioroebe/fasta/parse_fasta/parse_fasta.rb' 
      hash = parse_fasta_quietly(sequences.first).hash?
      sequences = hash.values
    end
    # ======================================================================= #
    # First obtain a handle towards the shortest sequence.
    # ======================================================================= #
    shortest_sequence = sequences.min_by(&:length)
    maxlen = shortest_sequence.length
    maxlen.downto(1) {|len|
      0.upto(maxlen - len) { |start|
        substring = shortest_sequence.to_s[start,len]
        if sequences.all? {|seq| seq.to_s.include? substring }
          return shortest_sequence.class.new(substring)
        end
      }
    }
    return nil
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  if ARGV.empty?
    e Bioroebe.longest_common_substring(
      %w( GATTACA TAGACCA ATACA )
    )
  else
    e Bioroebe.longest_common_substring(ARGV)
  end
end # longestcommonsubstring
    # longestcommonsubstring ATTTTT ATTT
    # longestcommonsubstring /rosalind_lcsm.txt