#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/open_in_browser.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/project/project.rb'
  require 'bioroebe/toplevel_methods/esystem.rb'
  require 'bioroebe/requires/require_yaml.rb'
  require 'bioroebe/constants/files_and_directories.rb'

  # ========================================================================= #
  # === USE_THIS_BROWSER
  #
  # This constant specifies the default browser to use for the
  # bioroebe-project.
  # ========================================================================= #
  if File.exist? "#{project_yaml_directory?}browser.yml"
    USE_THIS_BROWSER = YAML.load_file(FILE_BROWSER)
  elsif ENV['BROWSER']
    USE_THIS_BROWSER = ENV['BROWSER'].to_s.dup
  else
    USE_THIS_BROWSER = 'firefox' # opera # Hardcoded value in this case..
  end

  # ========================================================================= #
  # === Bioroebe.open_in_browser
  #
  # This method will try to open a remote URL in the specified browser -
  # the one that has been noted down in 'browser.yml'. It is accessed
  # via the Symbol :use_the_default_browser (as second argument to
  # this method).
  #
  # If you wish to make use of another browser then you can also
  # pass in the location to the browser as the second argument tp
  # this method.
  #
  # Usage example:
  #
  #   Bioroebe.open_in_browser('derstandard.at')
  #
  # ========================================================================= #
  def self.open_in_browser(
      this_url,
      use_this_browser = :use_the_default_browser
    )
    if this_url.is_a? Array
      this_url.each {|entry|
        open_in_browser(entry, use_this_browser)
      }
    else
      case use_this_browser # case tag
      # ===================================================================== #
      # === :default
      # ===================================================================== #
      when :default,
           :use_the_default_browser
        # =================================================================== #
        # Sanitize the default setting here, by loading up the content of
        # the proper yaml file.
        # =================================================================== #
        use_this_browser =
          YAML.load_file(FILE_BROWSER)
      end
      esystem "#{use_this_browser.strip} -new-tab #{this_url}"
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.open_in_browser 'https://www.youtube.com/'
end # rb open_in_browser.rb