#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/palindromes.rb'
# puts Bioroebe.is_palindrome? ARGV
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/e.rb'

  # ========================================================================= #
  # === Bioroebe.is_palindrome?
  #
  # This method will return true if the target sequence is a
  # palindrome, and false otherwise.
  #
  # When will a sequence be considered to be a "palindrome"?
  #
  # Firstly, it must have an even number of components. If it has an odd
  # number then it can never be a Palindrome, by definition alone.
  #
  # So, "GATC" is a palindrome, but "GATCC" can never be. (GATC is
  # a palindrome because the reverse of GATC will bind to itself
  # again.)
  #
  # Usage examples:
  #
  #   Bioroebe.is_palindrome? 'GAATTC'   # => true
  #   Bioroebe.is_palindrome? 'GAATTCCC' # => false
  #
  # ========================================================================= #
  def self.is_palindrome?(i)
    require 'bioroebe/toplevel_methods/nucleotides.rb'
    if i.is_a? Array
      i = i.first
    end
    hash = ::Bioroebe.partner_nucleotide_hash
    return_value = false
    if i
      if i.to_s.size.odd? # Then it can never be a Palindrome.
      else # Ok, it may be a palindrome here.
        first_half = i[0 .. (i.size / 2)-1]
        _ = ''.dup
        first_half.chars.reverse.each {|char|
          _ << hash[char] if hash.has_key? char
        }
        if (first_half + _) == i # Compare it to the original input.
          return_value = true
        end
      end
      return return_value
    else
      :invalid_input
    end
  end; self.instance_eval { alias is_this_sequence_a_palindrome? is_palindrome? } # === Bioroebe.is_this_sequence_a_palindrome?

end

if __FILE__ == $PROGRAM_NAME
  if ARGV.empty?
    p 'GAATTC: '+Bioroebe.is_palindrome?('GAATTC').to_s   # => true
    p 'GAATTCCC: '+Bioroebe.is_palindrome?('GAATTCCC').to_s # => false
  else
    p Bioroebe.is_palindrome? ARGV
  end
end # ispalindrome GATC # true
    # ispalindrome GAAG # false