#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/parse.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.parse
  #
  # This is a general parse-related method.
  #
  # The main ideafor this method is to parse files in particular, in a
  # simplified manner. That means, for example, if the input to this
  # method is a genbank file, then this method will simply return the 
  # sequence. If it is a .pdb file, same thing. A .fasta file, same
  # result - the content will be returned. Thus, this method attempts
  # to make it simpler to parse a wide array of different file
  # formats - simply feed it what you want and you get the
  # "most logical" output.
  # ========================================================================= #
  def self.parse(
      i = ARGV
    )
    if i.is_a? Array
      i = i.first
    end
    if i
      case i
      # ===================================================================== #
      # === .genbank
      # ===================================================================== #
      when /\.genbank$/i
        require 'bioroebe/fasta_and_fastq/parse_fasta/parse_fasta.rb' 
        i = Bioroebe.genbank_to_fasta(i, :be_quiet)
      # ===================================================================== #
      # === .pdb
      # ===================================================================== #
      when /\.pdb$/i
        require 'bioroebe/pdb/parse_pdb_file.rb'
        i = Bioroebe.return_aminoacid_sequence_from_this_pdb_file(i)
      # ===================================================================== #
      # === .fasta
      # ===================================================================== #
      when /\.fasta$/i,
           /\.fa$/i
        require 'bioroebe/fasta_and_fastq/parse_fasta/parse_fasta.rb'
        i = Bioroebe::ParseFasta.new(i) { :be_quiet }.sequence?
      end
      return i
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.parse(ARGV)
end # bioroebeparse