#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/return_subsequence_based_on_indices.rb'
# Bioroebe.return_subsequence_based_on_indices
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.return_subsequence_based_on_indices
  #
  # This method will return a subsequence that was sliced out from the
  # given input array of indices (the second argument to this method).
  #
  # This method here also allows us to read from an existing File. The
  # format of that file has to be simple, though - the first line must
  # be the sequence; and the second line will constitute the array
  # of indices.
  #
  # For a usage example, see the end of this file here please. It has
  # been written primarily to solve a specific task at Rosalind.
  # ========================================================================= #
  def self.return_subsequence_based_on_indices(
      i, *array_of_indices
    )
    if array_of_indices.is_a? Array
      array_of_indices.map! {|entry| entry.to_i }
    end
    result = ''.dup
    if i and i.is_a?(String) and File.exist?(i)
      dataset = File.readlines(i).reject {|line| line.strip.empty? }
      i = dataset.first
      array_of_indices = dataset.last.split(' ').map(&:to_i)
    end
    array_of_indices.each_slice(2).each {|a, b|
      result << i[a .. b]+' '
    }
    result.strip
  end

end

if __FILE__ == $PROGRAM_NAME
  a =  22 # These four variables determine the sub-indices.
  b =  27
  c =  97
  d = 102
  if ARGV.empty?
    test_this_string = 'HumptyDumptysatonawallHumptyDumpty'\
                       'hadagreatfallAlltheKingshorsesandalltheKingsmen'\
                       'CouldntputHumptyDumptyinhisplaceagain.'
  else
    test_this_string = ARGV.first
    if test_this_string and File.file?(test_this_string)
      dataset = File.readlines(test_this_string)
      test_this_string = dataset.first
      splitted = dataset.last.split(' ')
      a = splitted[0]
      b = splitted[1]
      c = splitted[2]
      d = splitted[3]
    end
  end
  puts Bioroebe.return_subsequence_based_on_indices(
    test_this_string, a, b, c, d
  )
end # returnsubsequencebasedonindices /HUM.txt