#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/rnalfold.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/e.rb'

  # ========================================================================= #
  # === Bioroebe.rnalfold
  #
  # This method allows us to make use of RNALfold. By default the method
  # will attempt to obtain all .fa files in the current directory; this
  # should simplify batch-processing.
  #
  # Otherwise, the first input argument to this method should be the
  # file(s) that you wish to manipulate.
  #
  # A typical usage example may be:
  #
  #   Bioroebe.rnalfold { '--zscore=-3 -L 200' }
  #
  # ========================================================================= #
  def self.rnalfold(
      i = :use_all_fasta_files_in_the_current_directory
    )
    require 'bioroebe/colours/sfancy.rb'
    require 'bioroebe/colours/sfile.rb'
    # ======================================================================= #
    # RNALfold will be used (as command).
    # ======================================================================= #
    use_this_command = 'RNALfold'.dup
    case i
    when :default,
         :use_all_fasta_files_in_the_current_directory
      i = Dir['**.fa']
    end
    if block_given?
      yielded = yield
      # ===================================================================== #
      # Simply append these block-commands in that case:
      # ===================================================================== #
      use_this_command << " #{yielded}"
    end
    # ======================================================================= #
    # We assume that our input is an Array past this point.
    # ======================================================================= #
    i.each {|this_file|
      e "Working on `#{sfancy(this_file)}` next."
      new_filename = this_file.dup
      regex_to_use = /-L (\d+)/
      use_this_command =~ regex_to_use
      new_filename << '.'+$1.to_s.dup
      new_filename << '.lfold'
      _ = use_this_command+' < '+this_file+' > '+new_filename
      esystem _
      if File.exist? new_filename
        e "A new file has been created at `#{sfile(new_filename)}`."
      end
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.rnalfold { '--zscore=-3 -L 200' }
end # bioroebernalfold