#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/shuffleseq.rb'
# Bioroebe.shuffleseq
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.shuffleseq
  #
  # This method simulates the Emboss shuffleseq-functionality. It will
  # take the input string, such as an assumed DNA sequence, and simply
  # re-arrange it as-is.
  #
  # You can read this here:
  #
  #   https://www.bioinformatics.nl/cgi-bin/emboss/shuffleseq
  #
  # Usage example:
  #
  #   Bioroebe.shuffleseq 'ATCGATCG' # => "AGTTACGC"
  #
  # ========================================================================= #
  def self.shuffleseq(i = ARGV)
    if i.is_a? Array
      i = i.join(' ').strip
    end
    return i.chars.shuffle.join
  end; self.instance_eval { alias randomize_the_sequence shuffleseq } # === Bioroebe.randomize_the_sequence

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.shuffleseq(ARGV)
end # shuffleseq ATCGATCG