#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/statistics.rb'
# Bioroebe.show_exon_statistics
# =========================================================================== #
module Bioroebe

  require 'bioroebe/colours/colours.rb'
  require 'bioroebe/toplevel_methods/e.rb'
  require 'bioroebe/toplevel_methods/esystem.rb'
  require 'bioroebe/toplevel_methods/file_and_directory_related_actions.rb'

  # ========================================================================= #
  # === Bioroebe.show_exon_statistics
  #
  # This is mostly "throw-away" code; it will quickly tell us how many
  # exons may span over some junction.
  #
  # It was specifically created to analyze a .gtf file.
  #
  # Invocation example:
  #
  #   bioroebe --show-exon-statistics-for=/tmp/praktikum/Mouse/chromosome_8/parsed/cdna.8.L100.global.gtf
  #
  # ========================================================================= #
  def self.show_exon_statistics(these_files)
    [these_files].flatten.compact.each {|this_file|
      if File.exist? this_file
        result = `cat #{this_file} | awk '{print $12}' | sort | uniq -c | sort -k1n | awk '{print $1}' | sort | uniq -c | less`
        e gold('        n exons')
        splitted = result.split("\n")
        # =================================================================== #
        # splitted may look like this:
        #
        #   ["  89839 1", "  26031 2", "   1136 3", "     24 4"]
        #
        # =================================================================== #
        splitted.each {|line|
          e line.sub(/( \d+)$/, gold('\1'))
        }
      else
        no_file_exists_at(this_file)
      end
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  Bioroebe.show_exon_statistics(ARGV)
end # showexonstatistics /tmp/praktikum/Mouse/chromosome_8/parsed/cdna.8.L100.global.gtf