#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# This file has some convenience method for simpler time and date support
# within the BioRoebe project.
# =========================================================================== #
# require 'bioroebe/toplevel_methods/time_and_date.rb'
# =========================================================================== #
module Bioroebe

  require 'bioroebe/toplevel_methods/e.rb'
  require 'bioroebe/toplevel_methods/verbose.rb'

  # ========================================================================= #
  # === Bioroebe.return_current_day_month_year
  # ========================================================================= #
  def self.return_current_day_month_year
    Time.now.strftime('%d.%m.%Y') # => "15.12.2020"
  end

  # ========================================================================= #
  # === Bioroebe.return_current_hours_minutes_seconds
  #
  # This method would return a String such as "21:03:32".
  # ========================================================================= #
  def self.return_current_hours_minutes_seconds
    Time.now.strftime('%H:%M:%S')
  end

  # ========================================================================= #
  # === Bioroebe.show_time_now
  #
  # This method will show the current time. It will do so only if the
  # verbosity-variable has been set to true.
  # ========================================================================= #
  def self.show_time_now(
      be_verbose = be_verbose?
    )
    require 'bioroebe/colours/rev.rb'
    require 'bioroebe/colours/simp.rb'
    @time = return_current_hours_minutes_seconds
    if be_verbose
      e "#{rev}Showing the current time: #{simp(@time)}"
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe.return_current_hours_minutes_seconds
  Bioroebe.show_time_now
end