#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/verbose.rb'
# =========================================================================== #
module Bioroebe

# =========================================================================== #
# === @be_verbose
#
# Keep track as to whether we want to be verbose or not, by default.
#
# Verbosity here refers to yielding output to the end user when
# something important has happened, such as a directory that was
# removed.
# =========================================================================== #
@be_verbose = true

# =========================================================================== #
# === Bioroebe.be_verbose?
# =========================================================================== #
def self.be_verbose?
  @be_verbose
end

# =========================================================================== #
# === Bioroebe::VerboseTruth
#
# We use a simplified approach for adding VerboseTruth supported to the
# BioRoebe project, by simply defining it as a submodule.
# =========================================================================== #
module VerboseTruth

  # ========================================================================= #
  # === Bioroebe::VerboseTruth[]
  # ========================================================================= #
  def self.[](i, append_dot = true)
    if i.is_a? Array
      i = i.first
    end
    result = ''.dup
    case i
    when 'true',
         true
      result << 'Yes'
    when 'false',
         false
      result << 'No'
    end
    result << '.' if append_dot
    result
  end

end; end

if __FILE__ == $PROGRAM_NAME
  puts Bioroebe::VerboseTruth[ARGV]
end # rb verbose.rb true