#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/blast.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.blast_neighborhood
  #
  # The second argument to this method is a score-filter, e. g. to
  # select only entries that have a score higher than 1.
  # ========================================================================= #
  def self.blast_neighborhood(
      this_mer                                    = 'CTC',
      optional_apply_filter_for_score_higher_than = nil
    )
    require 'bioroebe/toplevel_methods/e.rb'
    if this_mer.is_a? Array
      this_mer = this_mer.first
    end
    if this_mer.nil?
      this_mer = 'CTC' # Set the same default as above.
    end
    match_score     =  2
    mis_match_score = -2
    # ======================================================================= #
    # Next use an Array of sequences that we will compare.
    # ======================================================================= #
    compare_these_sequences = %w(
      AAA
      AAT
      AAC
      AAG
      ATA
      ATT
      ATC
      ATG
      ACA
      ACT
      ACC
      ACG
      AGA
      AGT
      AGC
      AGG
      TAA
      TAT
      TAC
      TAG
      TTA
      TTT
      TTC
      TTG
      TCA
      TCT
      TCC
      TCG
      TGA
      TGT
      TGC
      TGG
      CAA
      CAT
      CAC
      CAG
      CTA
      CTT
      CTC
      CTG
      CCA
      CCT
      CCC
      CCG
      CGA
      CGT
      CGC
      CGG
      GAA
      GAT
      GAC
      GAG
      GTA
      GTT
      GTC
      GTG
      GCA
      GCT
      GCC
      GCG
      GGA
      GGT
      GGC
      GGG
    )
    compare_these_sequences.each {|this_sequence|
      score = 0
      chars = this_sequence.chars
      first_char  = chars[0]
      second_char = chars[1]
      third_char  = chars[2]
      if first_char == this_mer[0]
        # =================================================================== #
        # Found the first match.
        # =================================================================== #
        score += match_score
      else
        # =================================================================== #
        # else it must be a mismatch
        # =================================================================== #
        score += mis_match_score
      end
      if second_char == this_mer[1]
        # =================================================================== #
        # Found the first match.
        # =================================================================== #
        score += match_score
      else
        # =================================================================== #
        # else it must be a mismatch
        # =================================================================== #
        score += mis_match_score
      end
      if third_char == this_mer[2]
        # =================================================================== #
        # Found the first match.
        # =================================================================== #
        score += match_score
      else
        # =================================================================== #
        # else it must be a mismatch
        # =================================================================== #
        score += mis_match_score
      end
      if optional_apply_filter_for_score_higher_than
         if (score.to_i > optional_apply_filter_for_score_higher_than)
          e "#{this_sequence}: score of "\
            "#{score.to_s.rjust(3)}"
         end
      else
        e this_sequence+': score of '+
          score.to_s.rjust(3)
      end
    }
  end

end

if __FILE__ == $PROGRAM_NAME
  # Bioroebe.blast_neighborhood(ARGV)
  Bioroebe.blast_neighborhood(ARGV, 1)
end # rb blast_neighb*