#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'bioroebe/toplevel_methods/map_ncbi_entry_to_eutils_id.rb'
# =========================================================================== #
module Bioroebe

  # ========================================================================= #
  # === Bioroebe.map_ncbi_entry_to_eutils_id
  #
  # This is a general "URL mapper" from certain NCBI IDs to specific
  # nucleotide sequences. Only some URLs will be mapped, in particular
  # those that I use more frequently.
  #
  # The method was created specifically to allow simpler input via
  # the commandline and the bioshell - it is easier to remember
  # a name such as "rhinovirus" as opposed to the NC entry, which
  # is "NC_038311".
  #
  # Usage examples:
  #
  #   Bioroebe.map_ncbi_entry_to_eutils_id 'rhinovirus'
  #   Bioroebe.map_ncbi_entry_to_eutils_id 'T6'
  # 
  # ========================================================================= #
  def self.map_ncbi_entry_to_eutils_id(
      i = 'NC_001416.1.fasta'
    )
    case i
    # ======================================================================= #
    # === Lambda phage genome
    # ======================================================================= #
    when /^-?-?NC_001416.1.fasta$/i,
         'lambda'
      i = '9626243'
    # ======================================================================= #
    # === P1
    # ======================================================================= #
    when 'NC_005856.1.fasta',
         'P1'
      i = '46401626'
    # ======================================================================= #
    # === P2
    # ======================================================================= #
    when 'NC_041848.1.fasta',
         'P2'
      i = '1631913463'
    # ======================================================================= #
    # === T12
    # ======================================================================= #
    when 'NC_028700.1.fasta',
         'T12'
      i = '966201481'
    # ======================================================================= #
    # === T2
    # ======================================================================= #
    when 'AP018813.1.fasta',
         'T2'
      i = 'AP018813.1'
    # ======================================================================= #
    # === T4
    # ======================================================================= #
    when 'NC_000866.4.fasta',
         'T4'
      i = 'NC_000866.4'
    # ======================================================================= #
    # === T6
    # ======================================================================= #
    when 'T6'
      i = 'MH550421.1'
    # ======================================================================= #
    # === rhinovirus
    # ======================================================================= #
    when /NC_038311/,
         'rhinovirus'
      i = '1464306962'
    end
    return i
  end

end

if __FILE__ == $PROGRAM_NAME
  alias e puts
  e Bioroebe.map_ncbi_entry_to_eutils_id 'rhinovirus'
  e Bioroebe.map_ncbi_entry_to_eutils_id 'T6'
end